﻿using System.Diagnostics;
using System.Reflection;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// ABOUT 대화 상자
    /// </summary>
    public class AboutDialog : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - AboutDialog(ownerWindow)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="ownerWindow">오너 윈도우</param>
        public AboutDialog(Window ownerWindow)
        {
            Assembly assembly = Assembly.GetExecutingAssembly();

            #region 제목

            AssemblyTitleAttribute assemblyTitleAttribute = (AssemblyTitleAttribute)assembly.GetCustomAttributes(typeof(AssemblyTitleAttribute), false)[0];

            string title = assemblyTitleAttribute.Title;

            #endregion
            #region 버전

            AssemblyFileVersionAttribute assemblyFileVersionAttribute = (AssemblyFileVersionAttribute)assembly.GetCustomAttributes(typeof(AssemblyFileVersionAttribute), false)[0];

            string version = assemblyFileVersionAttribute.Version.Substring(0, 3);

            #endregion
            #region 저작권

            AssemblyCopyrightAttribute assemblyCopyrightAttribute = (AssemblyCopyrightAttribute)assembly.GetCustomAttributes(typeof(AssemblyCopyrightAttribute), false)[0];

            string copyright = assemblyCopyrightAttribute.Copyright;

            #endregion

            Title         = "About " + title;
            ShowInTaskbar = false;
            SizeToContent = SizeToContent.WidthAndHeight;
            ResizeMode    = ResizeMode.NoResize;
            Left          = ownerWindow.Left + 96d;
            Top           = ownerWindow.Top  + 96d;

            StackPanel mainStackPanel = new StackPanel();

            Content = mainStackPanel;

            #region 제목 텍스트 블럭

            TextBlock titleTextBlock = new TextBlock();

            titleTextBlock.Text                = title + " Version " + version;
            titleTextBlock.FontFamily          = new FontFamily("Times New Roman");
            titleTextBlock.FontSize            = 32d;
            titleTextBlock.FontStyle           = FontStyles.Italic;
            titleTextBlock.Margin              = new Thickness(24d);
            titleTextBlock.HorizontalAlignment = HorizontalAlignment.Center;

            mainStackPanel.Children.Add(titleTextBlock);

            #endregion
            #region 저작권 텍스트 블럭

            TextBlock copyrightTextBlock = new TextBlock();

            copyrightTextBlock.Text                = copyright;
            copyrightTextBlock.FontSize            = 20; // 15 points.
            copyrightTextBlock.HorizontalAlignment = HorizontalAlignment.Center;

            mainStackPanel.Children.Add(copyrightTextBlock);

            #endregion
            #region 웹사이트 텍스트 블럭

            Run webSiteRun = new Run("www.charlespetzold.com");

            Hyperlink webSiteHyperlink = new Hyperlink(webSiteRun);

            webSiteHyperlink.Click += webSiteHyperlink_Click;

            TextBlock webSiteTextBlock = new TextBlock(webSiteHyperlink);

            webSiteTextBlock.FontSize            = 20;
            webSiteTextBlock.HorizontalAlignment = HorizontalAlignment.Center;

            mainStackPanel.Children.Add(webSiteTextBlock);

            #endregion
            #region OK 버튼

            Button okButton = new Button();

            okButton.Content             = "OK";
            okButton.IsDefault           = true;
            okButton.IsCancel            = true;
            okButton.HorizontalAlignment = HorizontalAlignment.Center;
            okButton.MinWidth            = 48d;
            okButton.Margin              = new Thickness(24d);

            okButton.Click += okButton_Click;

            mainStackPanel.Children.Add(okButton);

            okButton.Focus();

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 웹사이트 하이퍼링크 클릭시 처리하기 - webSiteHyperlink_Click(senser, e)

        /// <summary>
        /// 웹사이트 하이퍼링크 클릭시 처리하기
        /// </summary>
        /// <param name="senser">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void webSiteHyperlink_Click(object senser, RoutedEventArgs e)
        {
            Process.Start("http://www.charlespetzold.com");
        }

        #endregion
        #region OK 버튼 클릭시 처리하기 - okButton_Click(senser, e)

        /// <summary>
        /// OK 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="senser">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void okButton_Click(object senser, RoutedEventArgs e)
        {
            DialogResult = true;
        }

        #endregion
    }
}