﻿using System;
using System.Windows;
using System.Windows.Controls;

namespace TestProject
{
    /// <summary>
    /// 공통 대화 상자
    /// </summary>
    public abstract class CommonDialog : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 다음 찾기시 - FindNext

        /// <summary>
        /// 다음 찾기시
        /// </summary>
        public event EventHandler FindNext;

        #endregion
        #region 대체시 - Replace

        /// <summary>
        /// 대체시
        /// </summary>
        public event EventHandler Replace;

        #endregion
        #region 전체 대체시 - ReplaceAll

        /// <summary>
        /// 전체 대체시
        /// </summary>
        public event EventHandler ReplaceAll;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region Field

        /// <summary>
        /// 대체 레이블
        /// </summary>
        protected Label replaceLabel;

        /// <summary>
        /// 찾기 텍스트 박스
        /// </summary>
        protected TextBox findTextBox;

        /// <summary>
        /// 대체 텍스트 박스
        /// </summary>
        protected TextBox replaceTextBox;

        /// <summary>
        /// 대소문자 일치 체크 박스
        /// </summary>
        protected CheckBox matchCaseCheckBox;

        /// <summary>
        /// 찾기 방향 그룹 박스
        /// </summary>
        protected GroupBox findDirectionGroupBox;

        /// <summary>
        /// 아래 라디오 버튼
        /// </summary>
        protected RadioButton downRadioButton;

        /// <summary>
        /// 위쪽 라디오 버튼
        /// </summary>
        protected RadioButton upRadioButton;

        /// <summary>
        /// 찾기 버튼
        /// </summary>
        protected Button findButton;

        /// <summary>
        /// 대체 버튼
        /// </summary>
        protected Button replaceButton;

        /// <summary>
        /// 전체 대체 버튼
        /// </summary>
        protected Button replaceAllButton;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 찾을 텍스트 - FindText

        /// <summary>
        /// 찾을 텍스트
        /// </summary>
        public string FindText
        {
            set
            {
                this.findTextBox.Text = value;
            }
            get
            {
                return this.findTextBox.Text;
            }
        }

        #endregion
        #region 대체 텍스트 - ReplaceText 

        /// <summary>
        /// 대체 텍스트
        /// </summary>
        public string ReplaceText
        {
            set
            {
                this.replaceTextBox.Text = value;
            }
            get
            {
                return this.replaceTextBox.Text;
            }
        }

        #endregion
        #region 대소문자 일치 여부 - MatchCase

        /// <summary>
        /// 대소문자 일치 여부
        /// </summary>
        public bool MatchCase
        {
            set
            {
                this.matchCaseCheckBox.IsChecked = value;
            }
            get
            {
                return (bool)this.matchCaseCheckBox.IsChecked;
            }
        }

        #endregion
        #region 찾기 방향 - FindDirection

        /// <summary>
        /// 찾기 방향
        /// </summary>
        public FindDirection FindDirection
        {
            set
            {
                if(value == FindDirection.Down)
                {
                    this.downRadioButton.IsChecked = true;
                }
                else
                {
                    this.upRadioButton.IsChecked = true;
                }
            }
            get
            {
                return (bool)this.downRadioButton.IsChecked ? FindDirection.Down : FindDirection.Up;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 생성자 - CommonDialog(pOwnerWindow)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="pOwnerWindow">소유자 윈도우</param>
        protected CommonDialog(Window pOwnerWindow)
        {
            ShowInTaskbar         = false;
            WindowStyle           = WindowStyle.ToolWindow;
            SizeToContent         = SizeToContent.WidthAndHeight;
            WindowStartupLocation = WindowStartupLocation.CenterOwner;
            Owner                 = pOwnerWindow;

            Grid grid = new Grid();

            Content = grid;

            for(int i = 0; i < 3; i++)
            {
                RowDefinition rowDefinition = new RowDefinition();

                rowDefinition.Height = GridLength.Auto;

                grid.RowDefinitions.Add(rowDefinition);

                ColumnDefinition columnDefinition = new ColumnDefinition();

                columnDefinition.Width = GridLength.Auto;

                grid.ColumnDefinitions.Add(columnDefinition);
            }

            #region 찾기 레이블

            Label findLabel = new Label();

            findLabel.Content           = "Fi_nd what:";
            findLabel.VerticalAlignment = VerticalAlignment.Center;
            findLabel.Margin            = new Thickness(12);

            grid.Children.Add(findLabel);

            Grid.SetRow   (findLabel, 0);
            Grid.SetColumn(findLabel, 0);

            #endregion
            #region 찾기 텍스트 박스

            this.findTextBox = new TextBox();

            this.findTextBox.Margin = new Thickness(12);

            this.findTextBox.TextChanged += findTextBox_TextChanged;

            grid.Children.Add(this.findTextBox);

            Grid.SetRow   (this.findTextBox, 0);
            Grid.SetColumn(this.findTextBox, 1);

            #endregion
            #region 대체 레이블

            this.replaceLabel = new Label();

            this.replaceLabel.Content           = "Re_place with:";
            this.replaceLabel.VerticalAlignment = VerticalAlignment.Center;
            this.replaceLabel.Margin            = new Thickness(12);

            grid.Children.Add(this.replaceLabel);

            Grid.SetRow   (this.replaceLabel, 1);
            Grid.SetColumn(this.replaceLabel, 0);

            #endregion
            #region 대체 텍스트 박스

            this.replaceTextBox = new TextBox();

            this.replaceTextBox.Margin = new Thickness(12);

            grid.Children.Add(this.replaceTextBox);

            Grid.SetRow   (this.replaceTextBox, 1);
            Grid.SetColumn(this.replaceTextBox, 1);

            #endregion
            #region 대소문자 일치 체크 박스

            this.matchCaseCheckBox = new CheckBox();

            this.matchCaseCheckBox.Content           = "Match _case";
            this.matchCaseCheckBox.VerticalAlignment = VerticalAlignment.Center;
            this.matchCaseCheckBox.Margin            = new Thickness(12);

            grid.Children.Add(this.matchCaseCheckBox);

            Grid.SetRow   (this.matchCaseCheckBox, 2);
            Grid.SetColumn(this.matchCaseCheckBox, 0);

            #endregion
            #region 찾기 방향 그룹 박스

            this.findDirectionGroupBox = new GroupBox();

            this.findDirectionGroupBox.Header              = "Direction";
            this.findDirectionGroupBox.Margin              = new Thickness(12);
            this.findDirectionGroupBox.HorizontalAlignment = HorizontalAlignment.Left;

            grid.Children.Add(this.findDirectionGroupBox);

            Grid.SetRow   (this.findDirectionGroupBox, 2);
            Grid.SetColumn(this.findDirectionGroupBox, 1);

            #endregion
            #region 찾기 방향 스택 패널

            StackPanel findDirectionStackPanel = new StackPanel();

            findDirectionStackPanel.Orientation = Orientation.Horizontal;

            this.findDirectionGroupBox.Content = findDirectionStackPanel;

            #endregion
            #region 위쪽 라디오 버튼

            this.upRadioButton = new RadioButton();

            this.upRadioButton.Content = "_Up";
            this.upRadioButton.Margin  = new Thickness(6);

            findDirectionStackPanel.Children.Add(this.upRadioButton);

            #endregion
            #region 아래쪽 라디오 버튼

            this.downRadioButton = new RadioButton();

            this.downRadioButton.Content = "_Down";
            this.downRadioButton.Margin  = new Thickness(6);

            findDirectionStackPanel.Children.Add(this.downRadioButton);

            #endregion
            #region 버튼 스택 패널

            StackPanel buttonStackPanel = new StackPanel();

            buttonStackPanel.Margin = new Thickness(6);

            grid.Children.Add(buttonStackPanel);

            Grid.SetRow    (buttonStackPanel, 0);
            Grid.SetColumn (buttonStackPanel, 2);
            Grid.SetRowSpan(buttonStackPanel, 3);

            #endregion
            #region 찾기 버튼

            this.findButton = new Button();

            this.findButton.Content   = "_Find Next";
            this.findButton.Margin    = new Thickness(6);
            this.findButton.IsDefault = true;

            this.findButton.Click += findButton_Click;

            buttonStackPanel.Children.Add(this.findButton);

            #endregion
            #region 대체 버튼

            this.replaceButton = new Button();

            this.replaceButton.Content = "_Replace";
            this.replaceButton.Margin  = new Thickness(6);

            this.replaceButton.Click += replaceButton_Click;

            buttonStackPanel.Children.Add(this.replaceButton);

            #endregion
            #region 전부 대체 버튼

            this.replaceAllButton = new Button();

            this.replaceAllButton.Content = "Replace _All";
            this.replaceAllButton.Margin  = new Thickness(6);

            this.replaceAllButton.Click += replaceAllButton_Click;

            buttonStackPanel.Children.Add(this.replaceAllButton);

            #endregion
            #region 취소 버튼

            Button cancelButton = new Button();

            cancelButton.Content  = "Cancel";
            cancelButton.Margin   = new Thickness(6);
            cancelButton.IsCancel = true;

            cancelButton.Click += cancelButton_Click;

            buttonStackPanel.Children.Add(cancelButton);

            #endregion

            this.findTextBox.Focus();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 다음 찾기시 처리하기 - OnFindNext(e)

        /// <summary>
        /// 다음 찾기시 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected virtual void OnFindNext(EventArgs e)
        {
            FindNext?.Invoke(this, e);
        }

        #endregion
        #region 대체시 처리하기 - OnReplace(e)

        /// <summary>
        /// 대체시 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected virtual void OnReplace(EventArgs e)
        {
            Replace?.Invoke(this, e);
        }

        #endregion
        #region 전부 대체시 처리하기 - OnReplaceAll(e)

        /// <summary>
        /// 전부 대체시 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected virtual void OnReplaceAll(EventArgs e)
        {
            ReplaceAll?.Invoke(this, e);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 찾기 텍스트 박스 텍스트 변경시 처리하기 - findTextBox_TextChanged(sender, e)

        /// <summary>
        /// 찾기 텍스트 박스 텍스트 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void findTextBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            TextBox textBox = e.Source as TextBox;

            this.findButton.IsEnabled = this.replaceButton.IsEnabled =  this.replaceAllButton.IsEnabled = (textBox.Text.Length > 0);
        }

        #endregion
        #region 찾기 버튼 클릭시 처리하기 - findButton_Click(sender, e)

        /// <summary>
        /// 찾기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void findButton_Click(object sender, RoutedEventArgs e)
        {
            OnFindNext(new EventArgs());
        }

        #endregion
        #region 대체 버튼 클릭시 처리하기 - replaceButton_Click(sender, e)

        /// <summary>
        /// 대체 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void replaceButton_Click(object sender, RoutedEventArgs e)
        {
            OnReplace(new EventArgs());
        }

        #endregion
        #region 전부 대체 버튼 클릭시 처리하기 - replaceAllButton_Click.replaceAllButton_Click(sender, e)

        /// <summary>
        /// 전부 대체 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void replaceAllButton_Click(object sender, RoutedEventArgs e)
        {
            OnReplaceAll(new EventArgs());
        }

        #endregion
        #region 취소 버튼 클릭시 처리하기 - cancelButton_Click(sender, e)

        /// <summary>
        /// 취소 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void cancelButton_Click(object sender, RoutedEventArgs e)
        {
            Close();
        }

        #endregion
    }
}