﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace Petzold.ChooseFont
{
    /// <summary>
    /// 폰트 대화 상자
    /// </summary>
    public class FontDialog : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 패밀리 텍스트 리스터
        /// </summary>
        private TextLister familyTextLister;

        /// <summary>
        /// 스타일 텍스트 리스터
        /// </summary>
        private TextLister styleTextLister;

        /// <summary>
        /// 가중치 텍스트 리스터
        /// </summary>
        private TextLister weightTextLister;

        /// <summary>
        /// 확장 텍스트 리스터
        /// </summary>
        private TextLister stretchTextLister;

        /// <summary>
        /// 크기 텍스트 리스터
        /// </summary>
        private TextLister sizeTextLister;

        /// <summary>
        /// 출력 레이블
        /// </summary>
        private Label displayLabel;

        /// <summary>
        /// 샘플 텍스트 갱신 억제 여부
        /// </summary>
        private bool isSampleTextUpdateSuppressed = true;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 타입 페이스 - Typeface

        /// <summary>
        /// 타입 페이스
        /// </summary>
        public Typeface Typeface
        {
            set
            {
                if(this.familyTextLister.Contains(value.FontFamily))
                {
                    this.familyTextLister.SelectedItem  = value.FontFamily;
                }
                else
                {
                    this.familyTextLister.SelectedIndex = 0;
                }

                if(this.styleTextLister.Contains(value.Style))
                {
                    this.styleTextLister.SelectedItem  = value.Style;
                }
                else
                {
                    this.styleTextLister.SelectedIndex = 0;
                }

                if(this.weightTextLister.Contains(value.Weight))
                {
                    this.weightTextLister.SelectedItem  = value.Weight;
                }
                else
                {
                    this.weightTextLister.SelectedIndex = 0;
                }

                if(this.stretchTextLister.Contains(value.Stretch))
                {
                    this.stretchTextLister.SelectedItem  = value.Stretch;
                }
                else
                {
                    this.stretchTextLister.SelectedIndex = 0;
                }
            }
            get
            {
                return new Typeface
                (
                    (FontFamily)this.familyTextLister.SelectedItem,
                    (FontStyle)this.styleTextLister.SelectedItem,
                    (FontWeight)this.weightTextLister.SelectedItem,
                    (FontStretch)this.stretchTextLister.SelectedItem
                );
            }
        }

        #endregion
        #region 페이스 크기 - FaceSize

        /// <summary>
        /// 페이스 크기
        /// </summary>
        public double FaceSize
        {
            set
            {
                double size = 0.75d * value;

                this.sizeTextLister.Text = size.ToString();

                if(!this.sizeTextLister.Contains(size))
                {
                    this.sizeTextLister.Insert(0, size);
                }

                this.sizeTextLister.SelectedItem = size;
            }
            get
            {
                double size;

                if(!Double.TryParse(this.sizeTextLister.Text, out size))
                {
                    size = 8.25d;
                }

                return size / 0.75d;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - FontDialog()

        /// <summary>
        /// 생성자
        /// </summary>
        public FontDialog()
        {
            Title                 = "Font";
            ShowInTaskbar         = false;
            WindowStyle           = WindowStyle.ToolWindow;
            WindowStartupLocation = WindowStartupLocation.CenterOwner;
            SizeToContent         = SizeToContent.WidthAndHeight;
            ResizeMode            = ResizeMode.NoResize;


            Grid mainGrid = new Grid();

            Content = mainGrid;

            #region 메인 컨트롤 행 정의

            RowDefinition mainControlRowDefinition = new RowDefinition();

            mainControlRowDefinition.Height = new GridLength(200, GridUnitType.Pixel);

            mainGrid.RowDefinitions.Add(mainControlRowDefinition);

            #endregion
            #region 메인 샘플 텍스트 행 정의

            RowDefinition mainSampleTextRowDefinition = new RowDefinition();

            mainSampleTextRowDefinition.Height = new GridLength(150, GridUnitType.Pixel);

            mainGrid.RowDefinitions.Add(mainSampleTextRowDefinition);

            #endregion
            #region 메인 버튼 행 정의

            RowDefinition mainButtonRowDefinition = new RowDefinition();

            mainButtonRowDefinition.Height = GridLength.Auto;

            mainGrid.RowDefinitions.Add(mainButtonRowDefinition);

            #endregion
            #region 메인 컬럼 정의

            ColumnDefinition mainColumnDefinition = new ColumnDefinition();

            mainColumnDefinition.Width = new GridLength(650, GridUnitType.Pixel);

            mainGrid.ColumnDefinitions.Add(mainColumnDefinition);

            #endregion

            Grid controlGrid = new Grid();

            mainGrid.Children.Add(controlGrid);

            #region 컨트롤 레이블 행 정의

            RowDefinition controlLabelRowDefinition = new RowDefinition();

            controlLabelRowDefinition.Height = GridLength.Auto;

            controlGrid.RowDefinitions.Add(controlLabelRowDefinition);

            #endregion
            #region 컨트롤 컨트롤 행 정의

            RowDefinition controlControlRowDefinition = new RowDefinition();

            controlControlRowDefinition.Height = new GridLength(100, GridUnitType.Star);

            controlGrid.RowDefinitions.Add(controlControlRowDefinition);

            #endregion
            #region 컨트롤 패밀리 컬럼 정의

            ColumnDefinition controlFamilyColumnDefinition = new ColumnDefinition();

            controlFamilyColumnDefinition.Width = new GridLength(175, GridUnitType.Star);

            controlGrid.ColumnDefinitions.Add(controlFamilyColumnDefinition);

            #endregion
            #region 컨트롤 스타일 컬럼 정의

            ColumnDefinition controlStyleColumnDefinition = new ColumnDefinition();

            controlStyleColumnDefinition.Width = new GridLength(100, GridUnitType.Star);

            controlGrid.ColumnDefinitions.Add(controlStyleColumnDefinition);

            #endregion
            #region 컨트롤 가중치 컬럼 정의

            ColumnDefinition controlWeightColumnDefinition = new ColumnDefinition();

            controlWeightColumnDefinition.Width = new GridLength(100, GridUnitType.Star);

            controlGrid.ColumnDefinitions.Add(controlWeightColumnDefinition);

            #endregion
            #region 컨트롤 확장 컬럼 정의

            ColumnDefinition controlStretchColumnDefinition = new ColumnDefinition();

            controlStretchColumnDefinition.Width = new GridLength(100, GridUnitType.Star);

            controlGrid.ColumnDefinitions.Add(controlStretchColumnDefinition);

            #endregion
            #region 컨트롤 크기 컬럼 정의

            ColumnDefinition controlSizeColumnDefinition = new ColumnDefinition();

            controlSizeColumnDefinition.Width = new GridLength(75, GridUnitType.Star);

            controlGrid.ColumnDefinitions.Add(controlSizeColumnDefinition);

            #endregion
            #region 패밀리 레이블

            Label familyLabel = new Label();

            familyLabel.Content = "Font Family";
            familyLabel.Margin  = new Thickness(12, 12, 12, 0);

            controlGrid.Children.Add(familyLabel);

            Grid.SetRow   (familyLabel, 0);
            Grid.SetColumn(familyLabel, 0);

            #endregion
            #region 패밀리 텍스트 리스터

            this.familyTextLister = new TextLister();

            this.familyTextLister.IsReadOnly = true;
            this.familyTextLister.Margin     = new Thickness(12, 0, 12, 12);

            controlGrid.Children.Add(this.familyTextLister);

            Grid.SetRow   (this.familyTextLister, 1);
            Grid.SetColumn(this.familyTextLister, 0);

            #endregion
            #region 스타일 레이블

            Label styleLabel = new Label();

            styleLabel.Content = "Style";
            styleLabel.Margin  = new Thickness(12, 12, 12, 0);

            controlGrid.Children.Add(styleLabel);

            Grid.SetRow   (styleLabel, 0);
            Grid.SetColumn(styleLabel, 1);

            #endregion
            #region 스타일 텍스트 리스터

            this.styleTextLister = new TextLister();

            this.styleTextLister.IsReadOnly = true;
            this.styleTextLister.Margin     = new Thickness(12, 0, 12, 12);

            controlGrid.Children.Add(this.styleTextLister);

            Grid.SetRow   (this.styleTextLister, 1);
            Grid.SetColumn(this.styleTextLister, 1);

            #endregion
            #region 가중치 레이블

            Label weightLabel = new Label();

            weightLabel.Content = "Weight";
            weightLabel.Margin  = new Thickness(12, 12, 12, 0);

            controlGrid.Children.Add(weightLabel);

            Grid.SetRow   (weightLabel, 0);
            Grid.SetColumn(weightLabel, 2);

            #endregion
            #region 가중치 텍스트 리스터

            this.weightTextLister = new TextLister();

            this.weightTextLister.IsReadOnly = true;
            this.weightTextLister.Margin     = new Thickness(12, 0, 12, 12);

            controlGrid.Children.Add(this.weightTextLister);

            Grid.SetRow   (this.weightTextLister, 1);
            Grid.SetColumn(this.weightTextLister, 2);

            #endregion
            #region 확장 레이블

            Label stretchLabel = new Label();

            stretchLabel.Content = "Stretch";
            stretchLabel.Margin  = new Thickness(12, 12, 12, 0);

            controlGrid.Children.Add(stretchLabel);

            Grid.SetRow   (stretchLabel, 0);
            Grid.SetColumn(stretchLabel, 3);

            #endregion
            #region 확장 텍스트 리스터

            this.stretchTextLister = new TextLister();

            this.stretchTextLister.IsReadOnly = true;
            this.stretchTextLister.Margin     = new Thickness(12, 0, 12, 12);

            controlGrid.Children.Add(this.stretchTextLister);

            Grid.SetRow   (this.stretchTextLister, 1);
            Grid.SetColumn(this.stretchTextLister, 3);

            #endregion
            #region 크기 레이블

            Label sizeLabel = new Label();

            sizeLabel.Content = "Size";
            sizeLabel.Margin  = new Thickness(12, 12, 12, 0);

            controlGrid.Children.Add(sizeLabel);

            Grid.SetRow   (sizeLabel, 0);
            Grid.SetColumn(sizeLabel, 4);

            #endregion
            #region 크기 텍스트 리스터

            this.sizeTextLister = new TextLister();

            this.sizeTextLister.Margin = new Thickness(12, 0, 12, 12);

            controlGrid.Children.Add(this.sizeTextLister);

            Grid.SetRow   (this.sizeTextLister, 1);
            Grid.SetColumn(this.sizeTextLister, 4);

            #endregion
            #region 출력 레이블

            this.displayLabel = new Label();

            this.displayLabel.Content                    = "AaBbCc XxYzZz 012345";
            this.displayLabel.HorizontalContentAlignment = HorizontalAlignment.Center;
            this.displayLabel.VerticalContentAlignment   = VerticalAlignment.Center;

            mainGrid.Children.Add(this.displayLabel);

            Grid.SetRow(this.displayLabel, 1);

            #endregion

            Grid buttonGrid = new Grid();

            mainGrid.Children.Add(buttonGrid);

            Grid.SetRow(buttonGrid, 2);

            for(int i = 0; i < 5; i++)
            {
                buttonGrid.ColumnDefinitions.Add(new ColumnDefinition());
            }


            #region OK 버튼

            Button okButton = new Button();

            okButton.Content             = "OK";
            okButton.IsDefault           = true;
            okButton.HorizontalAlignment = HorizontalAlignment.Center;
            okButton.MinWidth            = 60;
            okButton.Margin              = new Thickness(12);

            okButton.Click += okButton_Click;

            buttonGrid.Children.Add(okButton);

            Grid.SetColumn(okButton, 1);

            #endregion
            #region Cancel 버튼

            Button cancelButton = new Button();

            cancelButton.Content             = "Cancel";
            cancelButton.IsCancel            = true;
            cancelButton.HorizontalAlignment = HorizontalAlignment.Center;
            cancelButton.MinWidth            = 60;
            cancelButton.Margin              = new Thickness(12);

            buttonGrid.Children.Add(cancelButton);

            Grid.SetColumn(cancelButton, 3);

            #endregion

            #region 패밀리 텍스트 리스터를 설정한다.

            foreach(FontFamily fontFamily in Fonts.SystemFontFamilies)
            {
                this.familyTextLister.Add(fontFamily);
            }

            #endregion
            #region 크기 텍스트 리스터를 설정한다.

            double[] sizeArray = new double[]
            {
                8, 9, 10, 11, 12, 14, 16, 18, 20, 22, 24, 26, 28, 36, 48, 72
            };

            foreach(double dize in sizeArray)
            {
                this.sizeTextLister.Add(dize);
            }

            #endregion

            this.familyTextLister.SelectionChanged  += familyTextLister_SelectionChanged;
            this.styleTextLister.SelectionChanged   += textLister_SelectionChanged;
            this.weightTextLister.SelectionChanged  += textLister_SelectionChanged;
            this.stretchTextLister.SelectionChanged += textLister_SelectionChanged;
            this.sizeTextLister.TextChanged         += sizeTextLister_TextChanged;

            Typeface = new Typeface(FontFamily, FontStyle, FontWeight, FontStretch);

            FaceSize = FontSize;

            this.familyTextLister.Focus();

            this.isSampleTextUpdateSuppressed = false;

            UpdateSample();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 패밀리 텍스트 리스터 선택 변경시 처리하기 - this._pFamilyTextLister_SelectionChanged(sender, e)

        /// <summary>
        /// 패밀리 텍스트 리스터 선택 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void familyTextLister_SelectionChanged(object sender, EventArgs e)
        {
            FontFamily fontFamily = (FontFamily)this.familyTextLister.SelectedItem;

            FontStyle?   previousFontStyle   = (FontStyle?)this.styleTextLister.SelectedItem;
            FontWeight?  previousFontWeight  = (FontWeight?)this.weightTextLister.SelectedItem;
            FontStretch? previousFontStretch = (FontStretch?)this.stretchTextLister.SelectedItem;

            this.isSampleTextUpdateSuppressed = true;

            this.styleTextLister.Clear();
            this.weightTextLister.Clear();
            this.stretchTextLister.Clear();

            foreach(FamilyTypeface familyTypeface in fontFamily.FamilyTypefaces)
            {
                if(!this.styleTextLister.Contains(familyTypeface.Style))
                {
                    if(familyTypeface.Style == FontStyles.Normal)
                    {
                        this.styleTextLister.Insert(0, familyTypeface.Style);
                    }
                    else
                    {
                        this.styleTextLister.Add(familyTypeface.Style);
                    }
                }

                if(!this.weightTextLister.Contains(familyTypeface.Weight))
                {
                    if(familyTypeface.Weight == FontWeights.Normal)
                    {
                        this.weightTextLister.Insert(0, familyTypeface.Weight);
                    }
                    else
                    {
                        this.weightTextLister.Add(familyTypeface.Weight);
                    }
                }

                if(!this.stretchTextLister.Contains(familyTypeface.Stretch))
                {
                    if(familyTypeface.Stretch == FontStretches.Normal)
                    {
                        this.stretchTextLister.Insert(0, familyTypeface.Stretch);
                    }
                    else
                    {
                        this.stretchTextLister.Add(familyTypeface.Stretch);
                    }
                }
            }

            if(this.styleTextLister.Contains(previousFontStyle))
            {
                this.styleTextLister.SelectedItem  = previousFontStyle;
            }
            else
            {
                this.styleTextLister.SelectedIndex = 0;
            }

            if(this.weightTextLister.Contains(previousFontWeight))
            {
                this.weightTextLister.SelectedItem  = previousFontWeight;
            }
            else
            {
                this.weightTextLister.SelectedIndex = 0;
            }
            if(this.stretchTextLister.Contains(previousFontStretch))
            {
                this.stretchTextLister.SelectedItem  = previousFontStretch;
            }
            else
            {
                this.stretchTextLister.SelectedIndex = 0;
            }

            this.isSampleTextUpdateSuppressed = false;

            UpdateSample();
        }

        #endregion
        #region 텍스트 리스터 선택 변경시 처리하기 - textLister_SelectionChanged(sender, e)

        /// <summary>
        /// 텍스트 리스터 선택 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void textLister_SelectionChanged(object sender, EventArgs e)
        {
            UpdateSample();
        }

        #endregion
        #region 크기 텍스트 리스터 텍스트 변경시 처리하기 - sizeTextLister_TextChanged(sender, e)

        /// <summary>
        /// 크기 텍스트 리스터 텍스트 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void sizeTextLister_TextChanged(object sender, TextChangedEventArgs e)
        {
            UpdateSample();
        }

        #endregion
        #region OK 버튼 클릭시 처리하기 - pOKButton_Click(sender, e)

        /// <summary>
        /// OK 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void okButton_Click(object sender, RoutedEventArgs e)
        {
            DialogResult = true;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 샘플 갱신하기 - UpdateSample()

        /// <summary>
        /// 샘플 갱신하기
        /// </summary>
        private void UpdateSample()
        {
            if(this.isSampleTextUpdateSuppressed)
            {
                return;
            }

            this.displayLabel.FontFamily  = (FontFamily)this.familyTextLister.SelectedItem;
            this.displayLabel.FontStyle   = (FontStyle)this.styleTextLister.SelectedItem;
            this.displayLabel.FontWeight  = (FontWeight)this.weightTextLister.SelectedItem;
            this.displayLabel.FontStretch = (FontStretch)this.stretchTextLister.SelectedItem;

            double size;

            if(!Double.TryParse(this.sizeTextLister.Text, out size))
            {
                size = 8.25d;
            }

            this.displayLabel.FontSize = size / 0.75d;
        }

        #endregion
    }
}