﻿using System;
using System.Collections;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace Petzold.ChooseFont
{
    /// <summary>
    /// 리스터
    /// </summary>
    public class Lister : ContentControl
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 선택 변경시 - SelectionChanged

        /// <summary>
        /// 선택 변경시
        /// </summary>
        public event EventHandler SelectionChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 스크롤 뷰어
        /// </summary>
        private ScrollViewer scrollViewer;

        /// <summary>
        /// 스택 패널
        /// </summary>
        private StackPanel stackPanel;

        /// <summary>
        /// 배열 리스트
        /// </summary>
        private ArrayList arrayList = new ArrayList();

        /// <summary>
        /// 선택 인덱스
        /// </summary>
        private int selectedIndex = -1;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 수 - Count

        /// <summary>
        /// 수
        /// </summary>
        public int Count
        {
            get
            {
                return this.arrayList.Count;
            }
        }

        #endregion
        #region 선택 인덱스 - SelectedIndex

        /// <summary>
        /// 선택 인덱스
        /// </summary>
        public int SelectedIndex
        {
            set
            {
                if(value < -1 || value >= Count)
                {
                    throw new ArgumentOutOfRangeException("SelectedIndex");
                }

                if(value == this.selectedIndex)
                {
                    return;
                }

                if(this.selectedIndex != -1)
                {
                    TextBlock textBlock = this.stackPanel.Children[this.selectedIndex] as TextBlock;

                    textBlock.Background = SystemColors.WindowBrush;
                    textBlock.Foreground = SystemColors.WindowTextBrush;
                }

                this.selectedIndex = value;

                if(this.selectedIndex > -1)
                {
                    TextBlock textBlock = this.stackPanel.Children[this.selectedIndex] as TextBlock;

                    textBlock.Background = SystemColors.HighlightBrush;
                    textBlock.Foreground = SystemColors.HighlightTextBrush;
                }

                ScrollIntoView();

                OnSelectionChanged(EventArgs.Empty);
            }
            get
            {
                return this.selectedIndex;
            }
        }

        #endregion
        #region 선택 항목 - SelectedItem

        /// <summary>
        /// 선택 항목
        /// </summary>
        public object SelectedItem
        {
            set
            {
                SelectedIndex = this.arrayList.IndexOf(value);
            }
            get
            {
                if(SelectedIndex > -1)
                {
                    return this.arrayList[SelectedIndex];
                }

                return null;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - Lister()

        /// <summary>
        /// 생성자
        /// </summary>
        public Lister()
        {
            Focusable = false;

            Border border = new Border();

            border.BorderThickness = new Thickness(1d);
            border.BorderBrush     = SystemColors.ActiveBorderBrush;
            border.Background      = SystemColors.WindowBrush;

            Content = border;

            this.scrollViewer = new ScrollViewer();

            this.scrollViewer.Focusable = false;
            this.scrollViewer.Padding   = new Thickness(2, 0, 0, 0);

            border.Child = this.scrollViewer;

            this.stackPanel = new StackPanel();

            this.scrollViewer.Content = this.stackPanel;

            AddHandler(TextBlock.MouseLeftButtonDownEvent, new MouseButtonEventHandler(textBlock_MouseLeftButtonDown));

            Loaded += ContentControl_Loaded;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 지우기 - Clear()

        /// <summary>
        /// 지우기
        /// </summary>
        public void Clear()
        {
            SelectedIndex = -1;

            this.stackPanel.Children.Clear();

            this.arrayList.Clear();
        }

        #endregion
        #region 추가하기 - Add(item)

        /// <summary>
        /// 추가하기
        /// </summary>
        /// <param name="item">항목</param>
        public void Add(object item)
        {
            this.arrayList.Add(item);

            TextBlock textBlock = new TextBlock();

            textBlock.Text = item.ToString();

            this.stackPanel.Children.Add(textBlock);
        }

        #endregion
        #region 삽입하기 - Insert(nIndex, item)

        /// <summary>
        /// 삽입하기
        /// </summary>
        /// <param name="index">인덱스</param>
        /// <param name="item">항목</param>
        public void Insert(int index, object item)
        {
            this.arrayList.Insert(index, item);

            TextBlock textBlock = new TextBlock();

            textBlock.Text = item.ToString();

            this.stackPanel.Children.Insert(index, textBlock);
        }

        #endregion
        #region 포함 여부 구하기 - Contains(item)

        /// <summary>
        /// 포함 여부 구하기
        /// </summary>
        /// <param name="item">항목</param>
        /// <returns>포함 여부</returns>
        public bool Contains(object item)
        {
            return this.arrayList.Contains(item);
        }

        #endregion
        #region 문자로 이동하기 - GoToLetter(character)

        /// <summary>
        /// 문자로 이동하기
        /// </summary>
        /// <param name="character">문자</param>
        public void GoToLetter(char character)
        {
            int offset = SelectedIndex + 1;

            for(int i = 0; i < Count; i++)
            {
                int index = (i + offset) % Count;

                if(Char.ToUpper(character) == Char.ToUpper(this.arrayList[index].ToString()[0]))
                {
                    SelectedIndex = index;

                    break;
                }
            }
        }

        #endregion
        #region 페이지 업 하기 - PageUp()

        /// <summary>
        /// 페이지 업 하기
        /// </summary>
        public void PageUp()
        {
            if(SelectedIndex == -1 || Count == 0)
            {
                return;
            }

            int index = SelectedIndex - (int)(Count * this.scrollViewer.ViewportHeight / this.scrollViewer.ExtentHeight);

            if(index < 0)
            {
                index = 0;
            }

            SelectedIndex = index;
        }

        #endregion
        #region 페이지 다운 하기 - PageDown()

        /// <summary>
        /// 페이지 다운 하기
        /// </summary>
        public void PageDown()
        {
            if(SelectedIndex == -1 || Count == 0)
            {
                return;
            }

            int index = SelectedIndex + (int)(Count * this.scrollViewer.ViewportHeight / this.scrollViewer.ExtentHeight);

            if(index > Count - 1)
            {
                index = Count - 1;
            }

            SelectedIndex = index;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 선택 변경시 처리하기 - OnSelectionChanged(e)

        /// <summary>
        /// 선택 변경시 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected virtual void OnSelectionChanged(EventArgs e)
        {
            SelectionChanged?.Invoke(this, e);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 컨텐트 컨트롤 로드시 처리하기 - ContentControl_Loaded(sender, e)

        /// <summary>
        /// 컨텐트 컨트롤 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void ContentControl_Loaded(object sender, RoutedEventArgs e)
        {
            ScrollIntoView();
        }

        #endregion
        #region 텍스트 블럭 마우스 왼쪽 버튼 다운시 처리하기 - pTextBlock_MouseLeftButtonDown(sender, e)

        /// <summary>
        /// 텍스트 블럭 마우스 왼쪽 버튼 다운시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void textBlock_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if(e.Source is TextBlock)
            {
                SelectedIndex = this.stackPanel.Children.IndexOf(e.Source as TextBlock);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 항목이 보이게 스크롤 하기 - ScrollIntoView()

        /// <summary>
        /// 항목이 보이게 스크롤 하기
        /// </summary>
        private void ScrollIntoView()
        {
            if(Count == 0 || SelectedIndex == -1 || this.scrollViewer.ViewportHeight > this.scrollViewer.ExtentHeight)
            {
                return;
            }

            double itemHeight       = this.scrollViewer.ExtentHeight / Count;
            double itemTopOffset    = SelectedIndex * itemHeight;
            double itemBottomOffset = (SelectedIndex + 1) * itemHeight;

            if(itemTopOffset < this.scrollViewer.VerticalOffset)
            {
                this.scrollViewer.ScrollToVerticalOffset(itemTopOffset);
            }
            else if(itemBottomOffset > this.scrollViewer.VerticalOffset + this.scrollViewer.ViewportHeight)
            {
                this.scrollViewer.ScrollToVerticalOffset(itemBottomOffset - this.scrollViewer.ViewportHeight);
            }
        }

        #endregion
    }
}