﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace TestProject
{
    /// <summary>
    /// 노트패드 클론 (Edit)
    /// </summary>
    public partial class NotepadClone
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region Undo 실행시 처리하기 - Undo_Executed(senser, e)

        /// <summary>
        /// Undo 실행시 처리하기
        /// </summary>
        /// <param name="senser">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Undo_Executed(object senser, ExecutedRoutedEventArgs e)
        {
            this.textBox.Undo();
        }

        #endregion
        #region Undo 실행 가능 여부 조사하기 - UndoCanExecute(senser, e)

        /// <summary>
        /// Undo 실행 가능 여부 조사하기
        /// </summary>
        /// <param name="senser">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void UndoCanExecute(object senser, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = this.textBox.CanUndo;
        }

        #endregion
        #region Redo 실행시 처리하기 - Redo_Executed(senser, e)

        /// <summary>
        /// Redo 실행시 처리하기
        /// </summary>
        /// <param name="senser">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Redo_Executed(object senser, ExecutedRoutedEventArgs e)
        {
            this.textBox.Redo();
        }

        #endregion
        #region Redo 실행 가능 여부 조사하기 - RedoCanExecute(senser, e)

        /// <summary>
        /// Redo 실행 가능 여부 조사하기
        /// </summary>
        /// <param name="senser">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void RedoCanExecute(object senser, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = this.textBox.CanRedo;
        }

        #endregion
        #region Cut 실행시 처리하기 - Cut_Executed(sender, e)

        /// <summary>
        /// Cut 실행시 처리하기
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e">이벤트 인자</param>
        private void Cut_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            this.textBox.Cut();
        }

        #endregion
        #region Cut 실행 가능 여부 조사하기 - CutCanExecute(senser, e)

        /// <summary>
        /// Cut 실행 가능 여부 조사하기
        /// </summary>
        /// <param name="senser">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void CutCanExecute(object senser, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = this.textBox.SelectedText.Length > 0;
        }

        #endregion
        #region Copy 실행시 처리하기 - Copy_Executed(senser, e)

        /// <summary>
        /// Copy 실행시 처리하기
        /// </summary>
        /// <param name="senser">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Copy_Executed(object senser, ExecutedRoutedEventArgs e)
        {
            this.textBox.Copy();
        }

        #endregion
        #region Delete 실행시 처리하기 - Delete_Executed(sender, e)

        /// <summary>
        /// Delete 실행시 처리하기
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e">이벤트 인자</param>
        private void Delete_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            this.textBox.SelectedText = string.Empty;
        }

        #endregion
        #region Paste 실행시 처리하기 - Paste_Executed(senser, e)

        /// <summary>
        /// Paste 실행시 처리하기
        /// </summary>
        /// <param name="senser">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Paste_Executed(object senser, ExecutedRoutedEventArgs e)
        {
            this.textBox.Paste();
        }

        #endregion
        #region Paste 실행 가능 여부 조사하기 - PasteCanExecute(senser, e)

        /// <summary>
        /// Paste 실행 가능 여부 조사하기
        /// </summary>
        /// <param name="senser">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void PasteCanExecute(object senser, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = Clipboard.ContainsText();
        }

        #endregion
        #region Select All 실행시 처리하기 - SelectAll_Executed(senser, e)

        /// <summary>
        /// Select All 실행시 처리하기
        /// </summary>
        /// <param name="senser">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void SelectAll_Executed(object senser, ExecutedRoutedEventArgs e)
        {
            this.textBox.SelectAll();
        }

        #endregion
        #region Time/Date 실행시 처리하기 - TimeDate_Executed(senser, e)

        /// <summary>
        /// Time/Date 실행시 처리하기
        /// </summary>
        /// <param name="senser">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void TimeDate_Executed(object senser, ExecutedRoutedEventArgs e)
        {
            this.textBox.SelectedText = DateTime.Now.ToString();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region Edit 메뉴 추가하기 - AddEditMenu(menu)

        /// <summary>
        /// Edit 메뉴 추가하기
        /// </summary>
        /// <param name="menu">Menu 객체</param>
        private void AddEditMenu(Menu menu)
        {
            #region Edit 메뉴 항목

            MenuItem editMenuItem = new MenuItem();

            editMenuItem.Header = "_Edit";

            menu.Items.Add(editMenuItem);

            #endregion
                #region Undo 메뉴 항목

                MenuItem undoMenuItem = new MenuItem();

                undoMenuItem.Header  = "_Undo";
                undoMenuItem.Command = ApplicationCommands.Undo;

                editMenuItem.Items.Add(undoMenuItem);

                CommandBindings.Add(new CommandBinding(ApplicationCommands.Undo, Undo_Executed, UndoCanExecute));

                #endregion
                #region Redo 메뉴 항목

                MenuItem redoMenuItem = new MenuItem();

                redoMenuItem.Header  = "_Redo";
                redoMenuItem.Command = ApplicationCommands.Redo;

                editMenuItem.Items.Add(redoMenuItem);

                CommandBindings.Add(new CommandBinding(ApplicationCommands.Redo, Redo_Executed, RedoCanExecute));

                #endregion

                editMenuItem.Items.Add(new Separator());

                #region Cut 메뉴 항목

                MenuItem cutMenuItem = new MenuItem();

                cutMenuItem.Header  = "Cu_t";
                cutMenuItem.Command = ApplicationCommands.Cut;

                editMenuItem.Items.Add(cutMenuItem);

                CommandBindings.Add(new CommandBinding(ApplicationCommands.Cut, Cut_Executed, CutCanExecute));

                #endregion
                #region Copy 메뉴 항목

                MenuItem copyMenuItem = new MenuItem();

                copyMenuItem.Header  = "_Copy";
                copyMenuItem.Command = ApplicationCommands.Copy;

                editMenuItem.Items.Add(copyMenuItem);

                CommandBindings.Add(new CommandBinding(ApplicationCommands.Copy, Copy_Executed, CutCanExecute));

                #endregion
                #region Paste 메뉴 항목

                MenuItem pasteMenuItem = new MenuItem();

                pasteMenuItem.Header  = "_Paste";
                pasteMenuItem.Command = ApplicationCommands.Paste;

                editMenuItem.Items.Add(pasteMenuItem);

                CommandBindings.Add(new CommandBinding(ApplicationCommands.Paste, Paste_Executed, PasteCanExecute));

                #endregion
                #region Delete 메뉴 항목

                MenuItem deleteMenuItem = new MenuItem();

                deleteMenuItem.Header  = "De_lete";
                deleteMenuItem.Command = ApplicationCommands.Delete;

                editMenuItem.Items.Add(deleteMenuItem);

                CommandBindings.Add(new CommandBinding(ApplicationCommands.Delete, Delete_Executed, CutCanExecute));

                #endregion

                editMenuItem.Items.Add(new Separator());

                AddFindMenuItems(editMenuItem);

                editMenuItem.Items.Add(new Separator());

                #region Select All 메뉴 항목

                MenuItem selectAllMenuItem = new MenuItem();

                selectAllMenuItem.Header  = "Select _All";
                selectAllMenuItem.Command = ApplicationCommands.SelectAll;

                editMenuItem.Items.Add(selectAllMenuItem);

                CommandBindings.Add(new CommandBinding(ApplicationCommands.SelectAll, SelectAll_Executed));

                #endregion
                #region Time/Date 메뉴 항목

                InputGestureCollection inputGestureCollection = new InputGestureCollection();

                inputGestureCollection.Add(new KeyGesture(Key.F5));

                RoutedUICommand timeDateRoutedUICommand = new RoutedUICommand("Time/_Date", "TimeDate", GetType(), inputGestureCollection);

                MenuItem timeDateMenuItem = new MenuItem();

                timeDateMenuItem.Command = timeDateRoutedUICommand;

                editMenuItem.Items.Add(timeDateMenuItem);

                CommandBindings.Add(new CommandBinding(timeDateRoutedUICommand, TimeDate_Executed));

                #endregion
        }

        #endregion
    }
}