﻿using Microsoft.Win32;
using System;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace TestProject
{
    /// <summary>
    /// 노트패드 클론 (File)
    /// </summary>
    public partial class NotepadClone : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region Field

        /// <summary>
        /// 필터
        /// </summary>
        protected string filter = "Text Documents(*.txt)|*.txt|All Files(*.*)|*.*";

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region New 실행시 처리하기 - New_Executed(sender, e)

        /// <summary>
        /// New 실행시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        protected virtual void New_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            if(!CheckToTrash())
            {
                return;
            }

            this.textBox.Text = string.Empty;
            this.filePath     = null;
            this.isFileDirty  = false;

            UpdateTitle();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region Open 실행시 처리하기 - Open_Executed(sender, e)

        /// <summary>
        /// Open 실행시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Open_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            if(!CheckToTrash())
            {
                return;
            }

            OpenFileDialog openFileDialog = new OpenFileDialog();

            openFileDialog.Filter = this.filter;

            if((bool)openFileDialog.ShowDialog(this))
            {
                LoadFile(openFileDialog.FileName);
            }
        }

        #endregion
        #region Save 실행시 처리하기 - Save_Executed(sender, e)

        /// <summary>
        /// Save 실행시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Save_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            if(this.filePath == null || this.filePath.Length == 0)
            {
                ShowSaveDialog(string.Empty);
            }
            else
            {
                SaveFile(this.filePath);
            }
        }

        #endregion
        #region Save As 실행시 처리하기 - SaveAs_Executed(sender, e)

        /// <summary>
        /// Save As 실행시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void SaveAs_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            ShowSaveDialog(this.filePath);
        }

        #endregion

        #region Exit 메뉴 항목 클릭시 처리하기 - exitMenuItem_Click(sender, e)

        /// <summary>
        /// Exit 메뉴 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void exitMenuItem_Click(object sender, RoutedEventArgs e)
        {
            Close();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region File 메뉴 추가하기 - AddFileMenu(menu)

        /// <summary>
        /// File 메뉴 추가하기
        /// </summary>
        /// <param name="menu">Menu 객체</param>
        private void AddFileMenu(Menu menu)
        {
            #region Field 메뉴 항목

            MenuItem fileMenuItem = new MenuItem();

            fileMenuItem.Header = "_File";

            menu.Items.Add(fileMenuItem);

            #endregion
                #region New 메뉴 항목

                MenuItem newMenuItem = new MenuItem();

                newMenuItem.Header  = "_New";
                newMenuItem.Command = ApplicationCommands.New;

                fileMenuItem.Items.Add(newMenuItem);

                CommandBindings.Add(new CommandBinding(ApplicationCommands.New, New_Executed));

                #endregion
                #region Open 메뉴 항목

                MenuItem openMenuItem = new MenuItem();

                openMenuItem.Header  = "_Open...";
                openMenuItem.Command = ApplicationCommands.Open;

                fileMenuItem.Items.Add(openMenuItem);

                CommandBindings.Add(new CommandBinding(ApplicationCommands.Open, Open_Executed));

                #endregion
                #region Save 메뉴 항목

                MenuItem saveMenuItem = new MenuItem();

                saveMenuItem.Header  = "_Save";
                saveMenuItem.Command = ApplicationCommands.Save;

                fileMenuItem.Items.Add(saveMenuItem);

                CommandBindings.Add(new CommandBinding(ApplicationCommands.Save, Save_Executed));

                #endregion
                #region Save As 메뉴 항목

                MenuItem saveAsMenuItem = new MenuItem();

                saveAsMenuItem.Header  = "Save _As...";
                saveAsMenuItem.Command = ApplicationCommands.SaveAs;

                fileMenuItem.Items.Add(saveAsMenuItem);

                CommandBindings.Add(new CommandBinding(ApplicationCommands.SaveAs, SaveAs_Executed));

                #endregion

                fileMenuItem.Items.Add(new Separator());

                AddPrintMenuItems(fileMenuItem);

                fileMenuItem.Items.Add(new Separator());

                #region Exit 메뉴 항목

                MenuItem exitMenuItem = new MenuItem();

                exitMenuItem.Header = "E_xit";

                exitMenuItem.Click += exitMenuItem_Click;

                fileMenuItem.Items.Add(exitMenuItem);

                #endregion
        }

        #endregion
        #region 저장 대화 상자 보여주기 - ShowSaveDialog(filePath)

        /// <summary>
        /// 저장 대화 상자 보여주기
        /// </summary>
        /// <param name="filePath">파일 경로</param>
        /// <returns>처리 결과</returns>
        private bool ShowSaveDialog(string filePath)
        {
            SaveFileDialog saveFileDialog = new SaveFileDialog();

            saveFileDialog.Filter   = this.filter;
            saveFileDialog.FileName = filePath;

            if((bool)saveFileDialog.ShowDialog(this))
            {
                SaveFile(saveFileDialog.FileName);

                return true;
            }

            return false;
        }

        #endregion
        #region 버리기 여부 조사하기 - CheckToTrash()

        /// <summary>
        /// 버리기 여부 조사하기
        /// </summary>
        /// <returns>버리기 여부</returns>
        private bool CheckToTrash()
        {
            if(!isFileDirty)
            {
                return true;
            }

            MessageBoxResult messageBoxResult = MessageBox.Show
            (
                "The text in the file " + this.filePath + " has changed\n\nDo you want to save the changes?",
                applicationTitle,
                MessageBoxButton.YesNoCancel,
                MessageBoxImage.Question,
                MessageBoxResult.Yes
            );

            if(messageBoxResult == MessageBoxResult.Cancel) // 취소한 경우
            {
                return false;
            }
            else if(messageBoxResult == MessageBoxResult.No) // 저장하지 않는 경우
            {
                return true;
            }
            else // 저장하는 경우
            {
                if(this.filePath != null && this.filePath.Length > 0)
                {
                    return SaveFile(this.filePath);
                }

                return ShowSaveDialog(string.Empty);
            }
        }

        #endregion
        #region 파일 로드하기 - LoadFile(filePath)

        /// <summary>
        /// 파일 로드하기
        /// </summary>
        /// <param name="filePath">파일 경로</param>
        private void LoadFile(string filePath)
        {
            try
            {
                this.textBox.Text = File.ReadAllText(filePath);
            }
            catch(Exception exception)
            {
                MessageBox.Show("Error on File Open : " + exception.Message, this.applicationTitle, MessageBoxButton.OK, MessageBoxImage.Asterisk);

                return;
            }

            this.filePath = filePath;

            UpdateTitle();

            this.textBox.SelectionStart  = 0;
            this.textBox.SelectionLength = 0;

            this.isFileDirty = false;
        }

        #endregion
        #region 파일 저장하기 - SaveFile(filePath)

        /// <summary>
        /// 파일 저장하기
        /// </summary>
        /// <param name="filePath">파일 경로</param>
        /// <returns>처리 결과</returns>
        private bool SaveFile(string filePath)
        {
            try
            {
                File.WriteAllText(filePath, this.textBox.Text);
            }
            catch(Exception exception)
            {
                MessageBox.Show("Error on File Save : " + exception.Message, this.applicationTitle, MessageBoxButton.OK, MessageBoxImage.Asterisk);

                return false;
            }

            this.filePath = filePath;

            UpdateTitle();

            this.isFileDirty = false;

            return true;
        }

        #endregion
    }
}