﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace TestProject
{
    /// <summary>
    /// 노트패드 클론 (Find)
    /// </summary>
    public partial class NotepadClone
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 찾기 텍스트
        /// </summary>
        private string findText = string.Empty;

        /// <summary>
        /// 대체 텍스트
        /// </summary>
        private string replaceText = string.Empty;

        /// <summary>
        /// 문자열 비교
        /// </summary>
        private StringComparison stringComparison = StringComparison.OrdinalIgnoreCase;

        /// <summary>
        /// 찾기 방향
        /// </summary>
        private FindDirection findDirection = FindDirection.Down;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 찾기 실행시 처리하기 - Find_Executed(sender, e)

        /// <summary>
        /// 찾기 실행시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Find_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            FindDialog findDialog = new FindDialog(this);

            findDialog.FindText      = this.findText;
            findDialog.MatchCase     = this.stringComparison == StringComparison.Ordinal;
            findDialog.FindDirection = this.findDirection;

            findDialog.FindNext += findDialog_FindNext;

            findDialog.Show();
        }

        #endregion
        #region 찾기 실행 가능 여부 조사하기 - FindCanExecute(sender, e)

        /// <summary>
        /// 찾기 실행 가능 여부 조사하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void FindCanExecute(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = (this.textBox.Text.Length > 0 && OwnedWindows.Count == 0);
        }

        #endregion
        #region 다음 찾기 실행시 처리하기 - FindNext_Executed(sender, e)

        /// <summary>
        /// 다음 찾기 실행시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void FindNext_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            if(this.findText == null || this.findText.Length == 0)
            {
                Find_Executed(sender, e);
            }
            else
            {
                FindNext();
            }
        }

        #endregion
        #region 다음 찾기 실행 가능 여부 조사하기 - FindNextCanExecute(sender, e)

        /// <summary>
        /// 다음 찾기 실행 가능 여부 조사하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void FindNextCanExecute(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = (this.textBox.Text.Length > 0 && this.findText.Length > 0);
        }

        #endregion
        #region 대체 실행시 처리하기 - Replace_Executed(sender, e)

        /// <summary>
        /// 대체 실행시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Replace_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            ReplaceDialog replaceDialog = new ReplaceDialog(this);

            replaceDialog.FindText      = this.findText;
            replaceDialog.ReplaceText   = this.replaceText;
            replaceDialog.MatchCase     = this.stringComparison == StringComparison.Ordinal;
            replaceDialog.FindDirection = this.findDirection;

            replaceDialog.FindNext   += findDialog_FindNext;
            replaceDialog.Replace    += replaceDialog_Replace;
            replaceDialog.ReplaceAll += replaceDialog_ReplaceAll;

            replaceDialog.Show();
        }

        #endregion

        #region 찾기 대화 상자 다음 찾기 처리하기 - findDialog_FindNext(sender, e)

        /// <summary>
        /// 찾기 대화 상자 다음 찾기 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void findDialog_FindNext(object sender, EventArgs e)
        {
            CommonDialog commonDialog = sender as CommonDialog;

            this.findText         = commonDialog.FindText;
            this.stringComparison = commonDialog.MatchCase ? StringComparison.Ordinal : StringComparison.OrdinalIgnoreCase;
            this.findDirection    = commonDialog.FindDirection;

            FindNext();
        }

        #endregion
        #region 대체 대화 상자 대체시 처리하기 - replaceDialog_Replace(sender, e)

        /// <summary>
        /// 대체 대화 상자 대체시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void replaceDialog_Replace(object sender, EventArgs e)
        {
            ReplaceDialog replaceDialog = sender as ReplaceDialog;

            this.findText         = replaceDialog.FindText;
            this.replaceText      = replaceDialog.ReplaceText;
            this.stringComparison = replaceDialog.MatchCase ? StringComparison.Ordinal : StringComparison.OrdinalIgnoreCase;

            if(this.findText.Equals(textBox.SelectedText, this.stringComparison))
            {
                this.textBox.SelectedText = this.replaceText;
            }

            FindNext();
        }

        #endregion
        #region 대체 대화 상자 전체 대체시 처리하기 - replaceDialog_ReplaceAll(sender, e)

        /// <summary>
        /// 대체 대화 상자 전체 대체시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void replaceDialog_ReplaceAll(object sender, EventArgs e)
        {
            ReplaceDialog replaceDialog = sender as ReplaceDialog;

            string text = textBox.Text;

            this.findText         = replaceDialog.FindText;
            this.replaceText      = replaceDialog.ReplaceText;
            this.stringComparison = replaceDialog.MatchCase ? StringComparison.Ordinal : StringComparison.OrdinalIgnoreCase;

            int index = 0;

            while(index + this.findText.Length < text.Length)
            {
                index = text.IndexOf(this.findText, index, this.stringComparison);

                if(index != -1)
                {
                    text = text.Remove(index, this.findText.Length);
                    text = text.Insert(index, this.replaceText);

                    index += this.replaceText.Length;
                }
                else
                {
                    break;
                }
            }

            this.textBox.Text = text;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region Find 메뉴 항목 추가하기 - AddFindMenuItems(editMenuItem)

        /// <summary>
        /// Find 메뉴 항목 추가하기
        /// </summary>
        /// <param name="editMenuItem">Edit 메뉴 항목</param>
        private void AddFindMenuItems(MenuItem editMenuItem)
        {
            #region Find 메뉴 항목

            MenuItem findMenuItem = new MenuItem();

            findMenuItem.Header  = "_Find...";
            findMenuItem.Command = ApplicationCommands.Find;

            editMenuItem.Items.Add(findMenuItem);

            CommandBindings.Add(new CommandBinding(ApplicationCommands.Find, Find_Executed, FindCanExecute));

            #endregion
            #region Find Next 메뉴 항목

            InputGestureCollection inputGestureCollection = new InputGestureCollection();

            inputGestureCollection.Add(new KeyGesture(Key.F3));

            RoutedUICommand findNextRoutedUICommand = new RoutedUICommand("Find _Next", "FindNext", GetType(), inputGestureCollection);

            MenuItem findNextMenuItem = new MenuItem();

            findNextMenuItem.Command = findNextRoutedUICommand;

            editMenuItem.Items.Add(findNextMenuItem);

            CommandBindings.Add(new CommandBinding(findNextRoutedUICommand, FindNext_Executed, FindNextCanExecute));

            #endregion
            #region Replace 메뉴 항목

            MenuItem replaceMenuItem = new MenuItem();

            replaceMenuItem.Header  = "_Replace...";
            replaceMenuItem.Command = ApplicationCommands.Replace;

            editMenuItem.Items.Add(replaceMenuItem);

            CommandBindings.Add(new CommandBinding(ApplicationCommands.Replace, Replace_Executed, FindCanExecute));

            #endregion
        }

        #endregion
        #region 다음 찾기 - FindNext()

        /// <summary>
        /// 다음 찾기
        /// </summary>
        private void FindNext()
        {
            int startIndex;
            int findIndex;

            if(this.findDirection == FindDirection.Down)
            {
                startIndex = this.textBox.SelectionStart + this.textBox.SelectionLength;
                findIndex  = this.textBox.Text.IndexOf(this.findText, startIndex, this.stringComparison);
            }
            else
            {
                startIndex = this.textBox.SelectionStart;
                findIndex  = this.textBox.Text.LastIndexOf(this.findText, startIndex, this.stringComparison);
            }

            if(findIndex != -1)
            {
                this.textBox.Select(findIndex, this.findText.Length);

                this.textBox.Focus();
            }
            else
            {
                MessageBox.Show("Cannot find \"" + this.findText + "\"", Title, MessageBoxButton.OK, MessageBoxImage.Information);
            }
        }

        #endregion
    }
}