﻿using System.Printing;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;

using Petzold.PrintWithMargins;

namespace TestProject
{
    /// <summary>
    /// 노트패드 클론 (Print)
    /// </summary>
    public partial class NotepadClone : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 프린트 큐
        /// </summary>
        private PrintQueue printQueue;

        /// <summary>
        /// 프린트 티켓
        /// </summary>
        private PrintTicket printTicket;

        /// <summary>
        /// 마진 두께
        /// </summary>
        private Thickness marginThickness = new Thickness(96);

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region Setup 메뉴 항목 클릭시 처리하기 - setupMenuItem_Click(sender, e)

        /// <summary>
        /// Setup 메뉴 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void setupMenuItem_Click(object sender, RoutedEventArgs e)
        {
            PageMarginDialog pageMarginDialog = new PageMarginDialog();

            pageMarginDialog.Owner      = this;
            pageMarginDialog.PageMargin = this.marginThickness;

            if(pageMarginDialog.ShowDialog().GetValueOrDefault())
            {
                this.marginThickness = pageMarginDialog.PageMargin;
            }
        }

        #endregion
        #region Print 메뉴 항목 클릭시 처리하기 - Print_Executed(sender, e)

        /// <summary>
        /// Print 메뉴 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Print_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            PrintDialog printDialog = new PrintDialog();

            if(this.printQueue != null)
            {
                printDialog.PrintQueue = this.printQueue;
            }

            if(this.printTicket != null)
            {
                printDialog.PrintTicket = this.printTicket;
            }

            if(printDialog.ShowDialog().GetValueOrDefault())
            {
                this.printQueue  = printDialog.PrintQueue;
                this.printTicket = printDialog.PrintTicket;

                PlainTextDocumentPaginator plainTextDocumentPaginator = new PlainTextDocumentPaginator();

                plainTextDocumentPaginator.PrintTicket = this.printTicket;
                plainTextDocumentPaginator.Text        = this.textBox.Text;
                plainTextDocumentPaginator.HeaderText  = this.filePath;
                plainTextDocumentPaginator.Typeface    = new Typeface(this.textBox.FontFamily, this.textBox.FontStyle, this.textBox.FontWeight, this.textBox.FontStretch);

                plainTextDocumentPaginator.EMSize       = this.textBox.FontSize;
                plainTextDocumentPaginator.TextWrapping = this.textBox.TextWrapping;
                plainTextDocumentPaginator.Margins      = this.marginThickness;
                plainTextDocumentPaginator.PageSize     = new Size(printDialog.PrintableAreaWidth, printDialog.PrintableAreaHeight);

                printDialog.PrintDocument(plainTextDocumentPaginator, Title);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region Print 메뉴 항목 추가하기 - AddPrintMenuItems(fileMenuItem)

        /// <summary>
        /// Print 메뉴 항목 추가하기
        /// </summary>
        /// <param name="fileMenuItem">File 메뉴 항목</param>
        private void AddPrintMenuItems(MenuItem fileMenuItem)
        {
            #region Setup 메뉴 항목

            MenuItem setupMenuItem = new MenuItem();

            setupMenuItem.Header = "Page Set_up...";

            setupMenuItem.Click += setupMenuItem_Click;

            fileMenuItem.Items.Add(setupMenuItem);

            #endregion
            #region Print 메뉴 항목
        
            MenuItem printMenuItem = new MenuItem();

            printMenuItem.Header  = "_Print...";
            printMenuItem.Command = ApplicationCommands.Print;

            fileMenuItem.Items.Add(printMenuItem);

            CommandBindings.Add(new CommandBinding(ApplicationCommands.Print, Print_Executed));

            #endregion
        }

        #endregion
    }
}