﻿using System;
using System.ComponentModel;
using System.IO;
using System.Reflection;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// 노트패드 클론
    /// </summary>
    public partial class NotepadClone : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region Field

        /// <summary>
        /// 애플리케이션 제목
        /// </summary>
        protected string applicationTitle;

        /// <summary>
        /// 설정 파일 경로
        /// </summary>
        protected string settingFilePath;

        /// <summary>
        /// 노트패드 클론 설정
        /// </summary>
        protected NotepadCloneSetting notepadCloneSetting;

        /// <summary>
        /// 파일 수정 여부
        /// </summary>
        protected bool isFileDirty = false;

        /// <summary>
        /// 메뉴
        /// </summary>
        protected Menu menu;

        /// <summary>
        /// 텍스트 박스
        /// </summary>
        protected TextBox textBox;

        /// <summary>
        /// 상태 바
        /// </summary>
        protected StatusBar statusBar;

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 파일 경로
        /// </summary>
        private string filePath;

        /// <summary>
        /// 상태 바 항목
        /// </summary>
        private StatusBarItem statusBarItem;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - NotepadClone()

        /// <summary>
        /// 생성자
        /// </summary>
        public NotepadClone()
        {
            Width      = 800;
            Height     = 600;
            FontFamily = new FontFamily("나눔고딕코딩");
            FontSize   = 16;

            Assembly assembly = Assembly.GetExecutingAssembly();

            #region 애플리케이션 제목

            AssemblyTitleAttribute assemblyTitleAttribute = (AssemblyTitleAttribute)assembly.GetCustomAttributes(typeof(AssemblyTitleAttribute), false)[0];

            this.applicationTitle = assemblyTitleAttribute.Title;

            #endregion

            AssemblyProductAttribute assemblyProductAttribute = (AssemblyProductAttribute)assembly.GetCustomAttributes(typeof(AssemblyProductAttribute), false)[0];

            this.settingFilePath = Path.Combine
            (
                Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData),
                "TEST\\" + assemblyProductAttribute.Product + "\\" + assemblyProductAttribute.Product + ".Settings.xml"
            );

            DockPanel dockPanel = new DockPanel();

            Content = dockPanel;

            this.menu = new Menu();

            this.menu.FontFamily = FontFamily;
            this.menu.FontSize   = FontSize;

            dockPanel.Children.Add(this.menu);

            DockPanel.SetDock(this.menu, Dock.Top);

            this.statusBar = new StatusBar();

            this.statusBar.FontFamily = FontFamily;
            this.statusBar.FontSize   = FontSize;

            dockPanel.Children.Add(this.statusBar);

            DockPanel.SetDock(this.statusBar, Dock.Bottom);

            this.statusBarItem = new StatusBarItem();

            this.statusBarItem.HorizontalAlignment = HorizontalAlignment.Right;

            this.statusBar.Items.Add(this.statusBarItem);

            this.textBox = new TextBox();

            this.textBox.AcceptsReturn                 = true;
            this.textBox.AcceptsTab                    = true;
            this.textBox.VerticalScrollBarVisibility   = ScrollBarVisibility.Auto;
            this.textBox.HorizontalScrollBarVisibility = ScrollBarVisibility.Auto;

            this.textBox.TextChanged      += textBox_TextChanged;
            this.textBox.SelectionChanged += textBox_SelectionChanged;

            dockPanel.Children.Add(this.textBox);

            AddFileMenu(this.menu);
            AddEditMenu(this.menu);
            AddFormatMenu(this.menu);
            AddViewMenu(this.menu);
            AddHelpMenu(this.menu);

            this.notepadCloneSetting = (NotepadCloneSetting)LoadSetting();

            WindowState = this.notepadCloneSetting.WindowState;

            if(this.notepadCloneSetting.RestoreBounds != Rect.Empty)
            {
                Left   = this.notepadCloneSetting.RestoreBounds.Left;
                Top    = this.notepadCloneSetting.RestoreBounds.Top;
                Width  = this.notepadCloneSetting.RestoreBounds.Width;
                Height = this.notepadCloneSetting.RestoreBounds.Height;
            }

            this.textBox.TextWrapping = this.notepadCloneSetting.TextWrapping;
            this.textBox.FontFamily   = new FontFamily(this.notepadCloneSetting.FontFamily);
            this.textBox.FontStyle    = (FontStyle)(new FontStyleConverter().ConvertFromString(this.notepadCloneSetting.FontStyle));
            this.textBox.FontWeight   = (FontWeight)(new FontWeightConverter().ConvertFromString(this.notepadCloneSetting.FontWeight));
            this.textBox.FontStretch  = (FontStretch)(new FontStretchConverter().ConvertFromString(this.notepadCloneSetting.FontStretch));
            this.textBox.FontSize     = this.notepadCloneSetting.FontSize;

            Loaded += Window_Loaded;

            this.textBox.Focus();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 닫을 경우 처리하기 - OnClosing(e)

        /// <summary>
        /// 닫을 경우 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnClosing(CancelEventArgs e)
        {
            base.OnClosing(e);

            e.Cancel = !CheckToTrash();

            this.notepadCloneSetting.RestoreBounds = RestoreBounds;
        }

        #endregion
        #region 닫는 경우 처리하기 - OnClosed(e)

        /// <summary>
        /// 닫는 경우 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnClosed(EventArgs e)
        {
            base.OnClosed(e);

            this.notepadCloneSetting.WindowState  = WindowState;
            this.notepadCloneSetting.TextWrapping = this.textBox.TextWrapping;
            this.notepadCloneSetting.FontFamily   = this.textBox.FontFamily.ToString();
            this.notepadCloneSetting.FontStyle    = new FontStyleConverter().ConvertToString(this.textBox.FontStyle);
            this.notepadCloneSetting.FontWeight   = new FontWeightConverter().ConvertToString(this.textBox.FontWeight);
            this.notepadCloneSetting.FontStretch  = new FontStretchConverter().ConvertToString(this.textBox.FontStretch);
            this.notepadCloneSetting.FontSize     = this.textBox.FontSize;

            SaveSetting();
        }

        #endregion

        #region 설정 로드하기 - LoadSetting()

        /// <summary>
        /// 설정 로드하기
        /// </summary>
        /// <returns>NotepadCloneSetting 객체</returns>
        protected virtual object LoadSetting()
        {
            return NotepadCloneSetting.Load(typeof(NotepadCloneSetting), this.settingFilePath);
        }

        #endregion
        #region 설정 저장하기 - SaveSetting()

        /// <summary>
        /// 설정 저장하기
        /// </summary>
        protected virtual void SaveSetting()
        {
            this.notepadCloneSetting.Save(this.settingFilePath);
        }

        #endregion

        #region 제목 갱신하기 - UpdateTitle()

        /// <summary>
        /// 제목 갱신하기
        /// </summary>
        protected void UpdateTitle()
        {
            if(this.filePath == null)
            {
                Title = "Untitled - " + this.applicationTitle;
            }
            else
            {
                Title = Path.GetFileName(this.filePath) + " - " + this.applicationTitle;
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 윈도우 로드시 처리하기 - Window_Loaded(sender, e)

        /// <summary>
        /// 윈도우 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            ApplicationCommands.New.Execute(null, this);

            string[] argumentArray = Environment.GetCommandLineArgs();

            if(argumentArray.Length > 1)
            {
                if(File.Exists(argumentArray[1]))
                {
                    LoadFile(argumentArray[1]);
                }
                else
                {
                    MessageBoxResult messageBoxResult = MessageBox.Show
                    (
                        "Cannot find the " + Path.GetFileName(argumentArray[1]) + " file.\r\n\r\n" + "Do you want to create a new file?",
                        this.applicationTitle,
                        MessageBoxButton.YesNoCancel,
                        MessageBoxImage.Question
                    );

                    if(messageBoxResult == MessageBoxResult.Cancel)
                    {
                        Close();
                    }
                    else if(messageBoxResult == MessageBoxResult.Yes)
                    {
                        try
                        {
                            File.Create(this.filePath = argumentArray[1]).Close();
                        }
                        catch(Exception exception)
                        {
                            MessageBox.Show
                            (
                                "Error on File Creation: " + exception.Message,
                                this.applicationTitle,
                                MessageBoxButton.OK,
                                MessageBoxImage.Asterisk
                            );

                            return;
                        }

                        UpdateTitle();
                    }
                }
            }
        }

        #endregion

        #region 텍스트 박스 텍스트 변경시 처리하기 - textBox_TextChanged(sender, e)

        /// <summary>
        /// 텍스트 박스 텍스트 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void textBox_TextChanged(object sender, RoutedEventArgs e)
        {
            this.isFileDirty = true;
        }

        #endregion
        #region 텍스트 박스 선택 변경시 처리하기 - textBox_SelectionChanged(sender, e)

        /// <summary>
        /// 텍스트 박스 선택 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void textBox_SelectionChanged(object sender, RoutedEventArgs e)
        {
            int nSelectionStart = this.textBox.SelectionStart;
            int nRow            = this.textBox.GetLineIndexFromCharacterIndex(nSelectionStart);

            if(nRow == -1)
            {
                this.statusBarItem.Content = string.Empty;

                return;
            }

            int    nColumn     = nSelectionStart - this.textBox.GetCharacterIndexFromLineIndex(nRow);
            string strLocation = String.Format("Line {0} Col {1}", nRow + 1, nColumn + 1);

            if(this.textBox.SelectionLength > 0)
            {
                nSelectionStart += this.textBox.SelectionLength;

                nRow = this.textBox.GetLineIndexFromCharacterIndex(nSelectionStart);

                nColumn = nSelectionStart - this.textBox.GetCharacterIndexFromLineIndex(nRow);

                strLocation += string.Format(" - Line {0} Col {1}", nRow + 1, nColumn + 1);
            }

            this.statusBarItem.Content = strLocation;
        }

        #endregion
    }
}