﻿using System;
using System.IO;
using System.Windows;
using System.Xml.Serialization;

namespace TestProject
{
    /// <summary>
    /// 노트패드 클론 설정
    /// </summary>
    public class NotepadCloneSetting
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 윈도우 상태
        /// </summary>
        public WindowState WindowState = WindowState.Normal;

        /// <summary>
        /// 복구 영역
        /// </summary>
        public Rect RestoreBounds = Rect.Empty;

        /// <summary>
        /// 텍스트 랩핑
        /// </summary>
        public TextWrapping TextWrapping = TextWrapping.NoWrap;

        /// <summary>
        /// 폰트 패밀리
        /// </summary>
        public string FontFamily = "";

        /// <summary>
        /// 폰트 스타일
        /// </summary>
        public string FontStyle = new FontStyleConverter().ConvertToString(FontStyles.Normal);

        /// <summary>
        /// 폰트 가중치
        /// </summary>
        public string FontWeight = new FontWeightConverter().ConvertToString(FontWeights.Normal);

        /// <summary>
        /// 폰트 확장
        /// </summary>
        public string FontStretch = new FontStretchConverter().ConvertToString(FontStretches.Normal);

        /// <summary>
        /// 폰트 크기
        /// </summary>
        public double FontSize = 11;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 저장하기 - Save(filePath)

        /// <summary>
        /// 저장하기
        /// </summary>
        /// <param name="filePath">파일 경로</param>
        /// <returns>처리 결과</returns>
        public virtual bool Save(string filePath)
        {
            try
            {
                Directory.CreateDirectory(Path.GetDirectoryName(filePath));

                StreamWriter streamWriter = new StreamWriter(filePath);

                XmlSerializer xmlSerializer = new XmlSerializer(GetType());

                xmlSerializer.Serialize(streamWriter, this);

                streamWriter.Close();
            }
            catch
            {
                return false;
            }

            return true;
        }

        #endregion
        #region 로드하기 - Load(objectType, filePath)

        /// <summary>
        /// 로드하기
        /// </summary>
        /// <param name="objectType">객체 타입</param>
        /// <param name="filePath">파일 경로</param>
        /// <returns>객체</returns>
        public static object Load(Type objectType, string filePath)
        {
            StreamReader streamReader;

            object targetObject;

            XmlSerializer xmlSerializer = new XmlSerializer(objectType);

            try
            {
                streamReader = new StreamReader(filePath);

                targetObject = xmlSerializer.Deserialize(streamReader);

                streamReader.Close();
            }
            catch
            {
                targetObject = objectType.GetConstructor(Type.EmptyTypes).Invoke(null);
            }

            return targetObject;
        }

        #endregion
    }
}