﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;

namespace Petzold.PrintWithMargins
{
    /// <summary>
    /// 페이지 마진 대화 상자
    /// </summary>
    public class PageMarginDialog : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Declaration
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Enumeration

        /// <summary>
        /// 측면 종류
        /// </summary>
        private enum SideType
        {
            /// <summary>
            /// 왼쪽
            /// </summary>
            Left,

            /// <summary>
            /// 오른쪽
            /// </summary>
            Right,

            /// <summary>
            /// 위쪽
            /// </summary>
            Top,

            /// <summary>
            /// 아래쪽
            /// </summary>
            Bottom
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 텍스트 블럭 배열
        /// </summary>
        private TextBox[] textBoxArray = new TextBox[4];

        /// <summary>
        /// OK 버튼
        /// </summary>
        private Button okButton;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 페이지 마진 - PageMargin

        /// <summary>
        /// 페이지 마진
        /// </summary>
        public Thickness PageMargin
        {
            set
            {
                this.textBoxArray[(int)SideType.Left  ].Text = (value.Left   / 96d).ToString("F3");
                this.textBoxArray[(int)SideType.Right ].Text = (value.Right  / 96d).ToString("F3");
                this.textBoxArray[(int)SideType.Top   ].Text = (value.Top    / 96d).ToString("F3");
                this.textBoxArray[(int)SideType.Bottom].Text = (value.Bottom / 96d).ToString("F3");
            }
            get
            {
                return new Thickness
                (
                    Double.Parse(this.textBoxArray[(int)SideType.Left  ].Text) * 96d,
                    Double.Parse(this.textBoxArray[(int)SideType.Right ].Text) * 96d,
                    Double.Parse(this.textBoxArray[(int)SideType.Top   ].Text) * 96d,
                    Double.Parse(this.textBoxArray[(int)SideType.Bottom].Text) * 96d
                );
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - PageMarginDialog()

        /// <summary>
        /// 생성자
        /// </summary>
        public PageMarginDialog()
        {
            Title                 = "Page Setup";
            ShowInTaskbar         = false;
            WindowStyle           = WindowStyle.ToolWindow;
            WindowStartupLocation = WindowStartupLocation.CenterOwner;
            SizeToContent         = SizeToContent.WidthAndHeight;
            ResizeMode            = ResizeMode.NoResize;

            StackPanel stackPanel = new StackPanel();

            Content = stackPanel;

            GroupBox groupBox = new GroupBox();

            groupBox.Header = "Margins (inches)";
            groupBox.Margin = new Thickness(12);

            stackPanel.Children.Add(groupBox);

            Grid grid = new Grid();

            grid.Margin = new Thickness(6);

            groupBox.Content = grid;

            for(int i = 0 ; i < 2; i ++)
            {
                RowDefinition rowDefinition = new RowDefinition();

                rowDefinition.Height = GridLength.Auto;

                grid.RowDefinitions.Add(rowDefinition);
            }

            for(int i = 0; i < 4; i++)
            {
                ColumnDefinition columnDefinition = new ColumnDefinition();

                columnDefinition.Width = GridLength.Auto;

                grid.ColumnDefinitions.Add(columnDefinition);
            }

            for(int i = 0; i < 4; i++)
            {
                Label label = new Label();

                label.Content           = "_" + Enum.GetName(typeof(SideType), i) + ":";
                label.Margin            = new Thickness(6);
                label.VerticalAlignment = VerticalAlignment.Center;

                grid.Children.Add(label);

                Grid.SetRow   (label, i / 2      );
                Grid.SetColumn(label, 2 * (i % 2));

                this.textBoxArray[i] = new TextBox();

                this.textBoxArray[i].TextChanged += textBox_TextChanged;

                this.textBoxArray[i].MinWidth = 48;
                this.textBoxArray[i].Margin   = new Thickness(6);

                grid.Children.Add(this.textBoxArray[i]);

                Grid.SetRow   (this.textBoxArray[i], i / 2          );
                Grid.SetColumn(this.textBoxArray[i], 2 * (i % 2) + 1);
            }

            UniformGrid uniformGrid = new UniformGrid();

            uniformGrid.Rows    = 1;
            uniformGrid.Columns = 2;

            stackPanel.Children.Add(uniformGrid);

            this.okButton = new Button();

            this.okButton.Content             = "OK";
            this.okButton.IsDefault           = true;
            this.okButton.IsEnabled           = false;
            this.okButton.MinWidth            = 60;
            this.okButton.Margin              = new Thickness(12);
            this.okButton.HorizontalAlignment = HorizontalAlignment.Center;

            this.okButton.Click += okButton_Click;

            uniformGrid.Children.Add(this.okButton);

            Button cancelButton = new Button();

            cancelButton.Content             = "Cancel";
            cancelButton.IsCancel            = true;
            cancelButton.MinWidth            = 60;
            cancelButton.Margin              = new Thickness(12);
            cancelButton.HorizontalAlignment = HorizontalAlignment.Center;

            uniformGrid.Children.Add(cancelButton);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 텍스트 박스 텍스트 변경시 처리하기 - textBox_TextChanged(sender, e)

        /// <summary>
        /// 텍스트 박스 텍스트 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void textBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            double result;

            this.okButton.IsEnabled = Double.TryParse(this.textBoxArray[(int)SideType.Left  ].Text, out result) &&
                                      Double.TryParse(this.textBoxArray[(int)SideType.Right ].Text, out result) &&
                                      Double.TryParse(this.textBoxArray[(int)SideType.Top   ].Text, out result) &&
                                      Double.TryParse(this.textBoxArray[(int)SideType.Bottom].Text, out result);
        }

        #endregion
        #region OK 버튼 클릭시 처리하기 - okButton_Click(sender, e)

        /// <summary>
        /// OK 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void okButton_Click(object sender, RoutedEventArgs e)
        {
            DialogResult = true;
        }

        #endregion
    }
}