﻿using System;
using System.Windows.Controls;
using System.Windows.Input;

namespace Petzold.ChooseFont
{
    /// <summary>
    /// 텍스트 리스터
    /// </summary>
    public class TextLister : ContentControl
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 텍스트 변경시 - TextChanged

        /// <summary>
        /// 텍스트 변경시
        /// </summary>
        public event TextChangedEventHandler TextChanged;

        #endregion
        #region 선택 변경시 - SelectionChanged

        /// <summary>
        /// 선택 변경시
        /// </summary>
        public event EventHandler SelectionChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 텍스트 박스
        /// </summary>
        private TextBox textBox;

        /// <summary>
        /// 리스터
        /// </summary>
        private Lister lister;

        /// <summary>
        /// 읽기 전용 여부
        /// </summary>
        private bool isReadOnly;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 텍스트 - Text

        /// <summary>
        /// 텍스트
        /// </summary>
        public string Text
        {
            set
            {
                this.textBox.Text = value;
            }
            get
            {
                return this.textBox.Text;
            }
        }

        #endregion
        #region 읽기 전용 여부 - IsReadOnly

        /// <summary>
        /// 읽기 전용 여부
        /// </summary>
        public bool IsReadOnly
        {
            set
            {
                this.isReadOnly = value;
            }
            get
            {
                return this.isReadOnly;
            }
        }

        #endregion
        #region 선택 인덱스 - SelectedIndex

        /// <summary>
        /// 선택 인덱스
        /// </summary>
        public int SelectedIndex
        {
            set
            {
                this.lister.SelectedIndex = value;

                if(this.lister.SelectedIndex == -1)
                {
                    this.textBox.Text = string.Empty;
                }
                else
                {
                    this.textBox.Text = this.lister.SelectedItem.ToString();
                }
            }
            get
            {
                return this.lister.SelectedIndex;
            }
        }

        #endregion
        #region 선택 항목 - SelectedItem

        /// <summary>
        /// 선택 항목
        /// </summary>
        public object SelectedItem
        {
            set
            {
                this.lister.SelectedItem = value;

                if(this.lister.SelectedItem != null)
                {
                    this.textBox.Text = this.lister.SelectedItem.ToString();
                }
                else
                {
                    this.textBox.Text = "";
                }
            }
            get
            {
                return this.lister.SelectedItem;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - TextLister()

        /// <summary>
        /// 생성자
        /// </summary>
        public TextLister()
        {
            DockPanel dockPanel = new DockPanel();

            Content = dockPanel;

            this.textBox = new TextBox();

            this.textBox.TextChanged += textBox_TextChanged;

            dockPanel.Children.Add(this.textBox);

            DockPanel.SetDock(this.textBox, Dock.Top);

            this.lister = new Lister();

            this.lister.SelectionChanged += lister_SelectionChanged;

            dockPanel.Children.Add(this.lister);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 추가하기 - Add(item)

        /// <summary>
        /// 추가하기
        /// </summary>
        /// <param name="item">항목</param>
        public void Add(object item)
        {
            this.lister.Add(item);
        }

        #endregion
        #region 삽입하기 - Insert(index, item)

        /// <summary>
        /// 삽입하기
        /// </summary>
        /// <param name="index">인덱스</param>
        /// <param name="item">항목</param>
        public void Insert(int index, object item)
        {
            this.lister.Insert(index, item);
        }

        #endregion
        #region 지우기 - Clear()

        /// <summary>
        /// 지우기
        /// </summary>
        public void Clear()
        {
            this.lister.Clear();
        }

        #endregion
        #region 포함 여부 구하기 - Contains(item)

        /// <summary>
        /// 포함 여부 구하기
        /// </summary>
        /// <param name="item">항목</param>
        /// <returns>포함 여부</returns>
        public bool Contains(object item)
        {
            return this.lister.Contains(item);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 마우스 다운시 처리하기 - OnMouseDown(e)

        /// <summary>
        /// 마우스 다운시 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnMouseDown(MouseButtonEventArgs e)
        {
            base.OnMouseDown(e);

            Focus();
        }

        #endregion
        #region 키보드 포커스 획득시 처리하기 - OnGotKeyboardFocus(e)

        /// <summary>
        /// 키보드 포커스 획득시 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnGotKeyboardFocus(KeyboardFocusChangedEventArgs e)
        {
            base.OnGotKeyboardFocus(e);

            if(e.NewFocus == this)
            {
                this.textBox.Focus();

                if(SelectedIndex == -1 && this.lister.Count > 0)
                {
                    SelectedIndex = 0;
                }
            }
        }

        #endregion
        #region 프리뷰 텍스트 입력시 처리하기 - OnPreviewTextInput(e)

        /// <summary>
        /// 프리뷰 텍스트 입력시 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnPreviewTextInput(TextCompositionEventArgs e)
        {
            base.OnPreviewTextInput(e);

            if(IsReadOnly)
            {
                this.lister.GoToLetter(e.Text[0]);

                e.Handled = true;
            }
        }

        #endregion
        #region 프리뷰 키 다운시 처리하기 - OnPreviewKeyDown(e)

        /// <summary>
        /// 프리뷰 키 다운시 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnPreviewKeyDown(KeyEventArgs e)
        {
            base.OnPreviewKeyDown(e);

            if(SelectedIndex == -1)
            {
                return;
            }

            switch(e.Key)
            {
                case Key.Home :

                    if(this.lister.Count > 0)
                    {
                        SelectedIndex = 0;
                    }

                    break;

                case Key.End :

                    if(this.lister.Count > 0)
                    {
                        SelectedIndex = this.lister.Count - 1;
                    }

                    break;

                case Key.Up :

                    if(SelectedIndex > 0)
                    {
                        SelectedIndex--;
                    }

                    break;

                case Key.Down :

                    if(SelectedIndex < this.lister.Count - 1)
                    {
                        SelectedIndex++;
                    }

                    break;

                case Key.PageUp :

                    this.lister.PageUp();

                    break;


                case Key.PageDown :

                    this.lister.PageDown();

                    break;

                default :

                    return;
            }

            e.Handled = true;
        }

        #endregion
        #region 선택 변경시 처리하기 - OnSelectionChanged(e)

        /// <summary>
        /// 선택 변경시 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected virtual void OnSelectionChanged(EventArgs e)
        {
            SelectionChanged?.Invoke(this, e);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 텍스트 박스 텍스트 변경시 처리하기 - textBox_TextChanged(sender, e)

        /// <summary>
        /// 텍스트 박스 텍스트 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void textBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            TextChanged?.Invoke(this, e);
        }

        #endregion
        #region 리스터 선택 변경시 처리하기 - lister_SelectionChanged(sender, e)

        /// <summary>
        /// 리스터 선택 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void lister_SelectionChanged(object sender, EventArgs e)
        {
            if(SelectedIndex == -1)
            {
                this.textBox.Text = string.Empty;
            }
            else
            {
                this.textBox.Text = this.lister.SelectedItem.ToString();
            }

            OnSelectionChanged(e);
        }

        #endregion
    }
}