﻿using System.Windows;
using System.Windows.Controls;

namespace TestProject
{
    /// <summary>
    /// 단어 랩핑 메뉴 항목
    /// </summary>
    public class WordWrapMenuItem : MenuItem
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 단어 랩핑
        /// </summary>
        public static DependencyProperty WordWrapProperty = DependencyProperty.Register("WordWrap", typeof(TextWrapping), typeof(WordWrapMenuItem));

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 단어 랩핑 - WordWrap

        /// <summary>
        /// 단어 랩핑
        /// </summary>
        public TextWrapping WordWrap
        {
            set
            {
                SetValue(WordWrapProperty, value);
            }
            get
            {
                return (TextWrapping)GetValue(WordWrapProperty);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - WordWrapMenuItem()

        /// <summary>
        /// 생성자
        /// </summary>
        public WordWrapMenuItem()
        {
            Header = "_Word Wrap";

            #region No Wrap 메뉴 항목

            MenuItem noWrapMenuItem = new MenuItem();

            noWrapMenuItem.Header = "_No Wrap";
            noWrapMenuItem.Tag    = TextWrapping.NoWrap;

            noWrapMenuItem.Click += menuItem_Click;

            Items.Add(noWrapMenuItem);

            #endregion
            #region Wrap 메뉴 항목

            MenuItem wrapMenuItem = new MenuItem();

            wrapMenuItem.Header = "_Wrap";
            wrapMenuItem.Tag    = TextWrapping.Wrap;

            wrapMenuItem.Click += menuItem_Click;

            Items.Add(wrapMenuItem);

            #endregion
            #region Wrap with Overflow 메뉴 항목

            MenuItem wrapWithOverflowMenuItem = new MenuItem();

            wrapWithOverflowMenuItem.Header = "Wrap with _Overflow";
            wrapWithOverflowMenuItem.Tag    = TextWrapping.WrapWithOverflow;

            wrapWithOverflowMenuItem.Click += menuItem_Click;

            Items.Add(wrapWithOverflowMenuItem);

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 하위 메뉴 오픈시 처리하기 - OnSubmenuOpened(e)

        /// <summary>
        /// 하위 메뉴 오픈시 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnSubmenuOpened(RoutedEventArgs e)
        {
            base.OnSubmenuOpened(e);

            foreach(MenuItem menuItem in Items)
            {
                menuItem.IsChecked = ((TextWrapping)menuItem.Tag == WordWrap);
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 메뉴 항목 클릭시 처리하기 - menuItem_Click(sender, e)

        /// <summary>
        /// 메뉴 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void menuItem_Click(object sender, RoutedEventArgs e)
        {
            WordWrap = (TextWrapping)(e.Source as MenuItem).Tag;
        }

        #endregion
    }
}