﻿using System;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;
using System.Windows.Markup;
using System.Windows.Media;
using System.Windows.Navigation;
using System.Windows.Threading;
using System.Xml;

namespace TestProject
{
    /// <summary>
    /// XAML Cruncher
    /// </summary>
    public class XamlCruncher : NotepadClone
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 프레임
        /// </summary>
        private Frame frame;

        /// <summary>
        /// 윈도우
        /// </summary>
        private Window window;

        /// <summary>
        /// 파싱 상태 바 항목
        /// </summary>
        private StatusBarItem parseStatusBarItem;

        /// <summary>
        /// 탭 스페이스 수
        /// </summary>
        private int tabSpaceCount = 4;

        /// <summary>
        /// XAML Cruncher 설정
        /// </summary>
        private XamlCruncherSetting xamlCruncherSetting;

        /// <summary>
        /// XAML 방향 메뉴 항목
        /// </summary>
        private XamlOrientationMenuItem xamlOrientationMenuItem;

        /// <summary>
        /// 파싱 억제 여부
        /// </summary>
        private bool isSuspendParsing = false;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 파싱 억제 여부 - IsSuspendParsing

        /// <summary>
        /// 파싱 억제 여부
        /// </summary>
        public bool IsSuspendParsing
        {
            set
            {
                this.isSuspendParsing = value;
            }
            get
            {
                return this.isSuspendParsing;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - XamlCruncher()

        /// <summary>
        /// 생성자
        /// </summary>
        public XamlCruncher()
        {
            filter = "XAML Files(*.xaml)|*.xaml|All Files(*.*)|*.*";

            DockPanel dockPanel = this.textBox.Parent as DockPanel;

            dockPanel.Children.Remove(this.textBox);

            Grid grid = new Grid();

            dockPanel.Children.Add(grid);

            for(int i = 0; i < 3; i++)
            {
                RowDefinition rowDefinition = new RowDefinition();

                rowDefinition.Height = new GridLength(0);

                grid.RowDefinitions.Add(rowDefinition);

                ColumnDefinition pColumnDefinition = new ColumnDefinition();

                pColumnDefinition.Width = new GridLength(0);

                grid.ColumnDefinitions.Add(pColumnDefinition);
            }

            grid.RowDefinitions[0].Height = new GridLength(100, GridUnitType.Star);

            grid.ColumnDefinitions[0].Width = new GridLength(100, GridUnitType.Star);

            GridSplitter horizontalGridSplitter = new GridSplitter();

            horizontalGridSplitter.HorizontalAlignment = HorizontalAlignment.Stretch;
            horizontalGridSplitter.VerticalAlignment   = VerticalAlignment.Center;
            horizontalGridSplitter.Height              = 6;

            grid.Children.Add(horizontalGridSplitter);

            Grid.SetRow       (horizontalGridSplitter, 1);
            Grid.SetColumn    (horizontalGridSplitter, 0);
            Grid.SetColumnSpan(horizontalGridSplitter, 3);

            GridSplitter verticalGridSplitter = new GridSplitter();

            verticalGridSplitter.HorizontalAlignment = HorizontalAlignment.Center;
            verticalGridSplitter.VerticalAlignment   = VerticalAlignment.Stretch;
            verticalGridSplitter.Width               = 6;

            grid.Children.Add(verticalGridSplitter);

            Grid.SetRow    (verticalGridSplitter, 0);
            Grid.SetColumn (verticalGridSplitter, 1);
            Grid.SetRowSpan(verticalGridSplitter, 3);

            this.frame = new Frame();

            this.frame.NavigationUIVisibility = NavigationUIVisibility.Hidden;

            grid.Children.Add(this.frame);

            this.textBox.TextChanged += textBox_TextChanged;

            grid.Children.Add(this.textBox);

            this.xamlCruncherSetting = (XamlCruncherSetting)notepadCloneSetting;

            #region Xaml 메뉴 항목

            MenuItem xamlMenuItem = new MenuItem();

            xamlMenuItem.Header = "_Xaml";

            menu.Items.Insert(menu.Items.Count - 1, xamlMenuItem);

            #endregion
                #region Orientation 메뉴 항목

                this.xamlOrientationMenuItem = new XamlOrientationMenuItem(grid, this.textBox, this.frame);

                this.xamlOrientationMenuItem.Orientation = this.xamlCruncherSetting.Orientation;

                xamlMenuItem.Items.Add(this.xamlOrientationMenuItem);

                #endregion
                #region Tab Spaces Count 메뉴 항목

                MenuItem tabSpaceCountMenuItem = new MenuItem();

                tabSpaceCountMenuItem.Header = "_Tab Space Count...";

                tabSpaceCountMenuItem.Click += tabSpaceCountMenuItem_Click;

                xamlMenuItem.Items.Add(tabSpaceCountMenuItem);

                #endregion
                #region Suspend Parsing 메뉴 항목

                MenuItem suspendParsingMenuItem = new MenuItem();

                suspendParsingMenuItem.Header      = "_Suspend Parsing";
                suspendParsingMenuItem.IsCheckable = true;

                suspendParsingMenuItem.SetBinding(MenuItem.IsCheckedProperty, "IsSuspendParsing");

                suspendParsingMenuItem.DataContext = this;

                xamlMenuItem.Items.Add(suspendParsingMenuItem);

                #endregion
                #region Reparse 메뉴 항목

                InputGestureCollection reparseInputGestureCollection = new InputGestureCollection();

                reparseInputGestureCollection.Add(new KeyGesture(Key.F6));

                RoutedUICommand reparseRoutedUICommand = new RoutedUICommand("_Reparse", "Reparse", GetType(), reparseInputGestureCollection);

                MenuItem reparseMenuItem = new MenuItem();

                reparseMenuItem.Command = reparseRoutedUICommand;

                xamlMenuItem.Items.Add(reparseMenuItem);

                CommandBindings.Add(new CommandBinding(reparseRoutedUICommand, Reparse_Executed));

                #endregion
                #region Show Window 메뉴 항목

                InputGestureCollection showWindowInputGestureCollection = new InputGestureCollection();

                showWindowInputGestureCollection.Add(new KeyGesture(Key.F7));

                RoutedUICommand showWindowRoutedUICommand = new RoutedUICommand("Show _Window", "ShowWindow", GetType(), showWindowInputGestureCollection);

                MenuItem showWindowMenuItem = new MenuItem();

                showWindowMenuItem.Command = showWindowRoutedUICommand;

                xamlMenuItem.Items.Add(showWindowMenuItem);

                CommandBindings.Add(new CommandBinding(showWindowRoutedUICommand, ShowWindow_Executed, ShowWindow_CanExecute));

                #endregion
                #region Template 메뉴 항목

                MenuItem templateMenuItem = new MenuItem();

                templateMenuItem.Header = "Save as Startup _Document";

                templateMenuItem.Click += pTemplateMenuItem_Click;

                xamlMenuItem.Items.Add(templateMenuItem);

                #endregion

            // Help 메뉴 항목

                #region Xaml Help 메뉴 항목

                MenuItem xamlHelpMenuItem = new MenuItem();

                xamlHelpMenuItem.Header = "_Help...";

                xamlHelpMenuItem.Click += helpMenuItem_Click;

                MenuItem helpMenuItem = (MenuItem)menu.Items[menu.Items.Count - 1];

                helpMenuItem.Items.Insert(0, xamlHelpMenuItem);

                #endregion

            this.parseStatusBarItem = new StatusBarItem();

            this.statusBar.Items.Insert(0, this.parseStatusBarItem);

            this.statusBar.Visibility = Visibility.Visible;

            Dispatcher.UnhandledException += Dispatcher_UnhandledException;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 프로그램 시작하기 - Main()

        /// <summary>
        /// 프로그램 시작하기
        /// </summary>
        [STAThread]
        public new static void Main()
        {
            Application application = new Application();

            application.ShutdownMode = ShutdownMode.OnMainWindowClose;

            application.Run(new XamlCruncher());
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Protected
        ////////////////////////////////////////////////////////////////////// Event

        #region New 실행시 처리하기 - New_Executed(sender, e)

        /// <summary>
        /// New 실행시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        protected override void New_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            base.New_Executed(sender, e);

            string startupDocument = ((XamlCruncherSetting)this.notepadCloneSetting).StartupDocument;

            startupDocument = startupDocument.Replace("\r\n", "\n");
            startupDocument = startupDocument.Replace("\n", "\r\n");

            this.textBox.Text = startupDocument;

            this.isFileDirty = false;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////// Function

        #region 닫는 경우 처리하기 - OnClosed(e)

        /// <summary>
        /// 닫는 경우 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnClosed(EventArgs e)
        {
            this.xamlCruncherSetting.Orientation = this.xamlOrientationMenuItem.Orientation;

            base.OnClosed(e);
        }

        #endregion
        #region 프리뷰 키 다운시 처리하기 - OnPreviewKeyDown(e)

        /// <summary>
        /// 프리뷰 키 다운시 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnPreviewKeyDown(KeyEventArgs e)
        {
            base.OnPreviewKeyDown(e);

            if(e.Source == textBox && e.Key == Key.Tab)
            {
                string tabSpaceString = new string(' ', this.tabSpaceCount);
                int    characterIndex = textBox.SelectionStart;
                int    lineIndex      = textBox.GetLineIndexFromCharacterIndex(characterIndex);

                if(lineIndex != -1)
                {
                    int columnIndex = characterIndex - textBox.GetCharacterIndexFromLineIndex(lineIndex);

                    tabSpaceString = new string(' ', this.xamlCruncherSetting.TabSpaceCount - columnIndex % this.xamlCruncherSetting.TabSpaceCount);
                }

                textBox.SelectedText = tabSpaceString;
                textBox.CaretIndex   = textBox.SelectionStart + textBox.SelectionLength;

                e.Handled = true;
            }
        }

        #endregion

        #region 설정 로드하기 - LoadSetting()

        /// <summary>
        /// 설정 로드하기
        /// </summary>
        /// <returns>설정 객체</returns>
        protected override object LoadSetting()
        {
            return XamlCruncherSetting.Load(typeof(XamlCruncherSetting), this.settingFilePath);
        }

        #endregion
        #region 설정 저장하기 - SaveSetting()

        /// <summary>
        /// 설정 저장하기
        /// </summary>
        protected override void SaveSetting()
        {
            ((XamlCruncherSetting)notepadCloneSetting).Save(this.settingFilePath);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private
        ////////////////////////////////////////////////////////////////////// Event

        #region 디스패처 다룰 수 없는 예외 발생시 처리하기 - Dispatcher_UnhandledException(sender, e)

        /// <summary>
        /// 디스패처 다룰 수 없는 예외 발생시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Dispatcher_UnhandledException(object sender, DispatcherUnhandledExceptionEventArgs e)
        {
            this.parseStatusBarItem.Content = "Unhandled Exception: " + e.Exception.Message;

            e.Handled = true;
        }

        #endregion
        #region 탭 스페이스 수 메뉴 항목 클릭시 처리하기 - tabSpaceCountMenuItem_Click(sender, e)

        /// <summary>
        /// 탭 스페이스 수 메뉴 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void tabSpaceCountMenuItem_Click(object sender, RoutedEventArgs e)
        {
            XamlTabSpaceCountDialog xamlTabSpaceCountDialog = new XamlTabSpaceCountDialog();

            xamlTabSpaceCountDialog.Owner         = this;
            xamlTabSpaceCountDialog.TabSpaceCount = this.xamlCruncherSetting.TabSpaceCount;

            if((bool)xamlTabSpaceCountDialog.ShowDialog().GetValueOrDefault())
            {
                this.xamlCruncherSetting.TabSpaceCount = xamlTabSpaceCountDialog.TabSpaceCount;
            }
        }

        #endregion
        #region Reparse 실행시 처리하기 - Reparse_Executed(sender, e)

        /// <summary>
        /// Reparse 실행시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Reparse_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            Parse();
        }

        #endregion
        #region Show Window 실행시 처리하기 - ShowWindow_Executed(sender, e)

        /// <summary>
        /// Show Window 실행시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void ShowWindow_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            if(this.window != null)
            {
                this.window.Show();
            }
        }

        #endregion
        #region Show Window 실행 가능 여부 조사하기 - ShowWindow_CanExecute(sender, e)

        /// <summary>
        /// Show Window 실행 가능 여부 조사하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void ShowWindow_CanExecute(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = (this.window != null);
        }

        #endregion
        #region Template 메뉴 항목 클릭시 처리하기 - pTemplateMenuItem_Click(sender, e)

        /// <summary>
        /// Template 메뉴 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void pTemplateMenuItem_Click(object sender, RoutedEventArgs e)
        {
            ((XamlCruncherSetting)this.notepadCloneSetting).StartupDocument = this.textBox.Text;
        }

        #endregion
        #region Help 메뉴 항목 클릭시 처리하기 - helpMenuItem_Click(sender, e)

        /// <summary>
        /// Help 메뉴 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void helpMenuItem_Click(object sender, RoutedEventArgs e)
        {
            Uri uri = new Uri("pack://application:,,,/XamlCruncherHelp.xaml");

            Stream stream = Application.GetResourceStream(uri).Stream;

            Window window = new Window();

            window.Title   = "XAML Cruncher Help";
            window.Content = XamlReader.Load(stream);

            window.Show();
        }

        #endregion
        #region 텍스트 박스 텍스트 변경시 처리하기 - textBox_TextChanged(sender, e)

        /// <summary>
        /// 텍스트 박스 텍스트 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void textBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            if(IsSuspendParsing)
            {
                this.textBox.Foreground = SystemColors.WindowTextBrush;
            }
            else
            {
                Parse();
            }
        }

        #endregion
        #region 파싱하기 - Parse()

        /// <summary>
        /// 파싱하기
        /// </summary>
        private void Parse()
        {
            StringReader  stringReader  = new StringReader(this.textBox.Text);
            XmlTextReader xmlTextReader = new XmlTextReader(stringReader);

            try
            {
                object targetObject = XamlReader.Load(xmlTextReader);

                this.textBox.Foreground = SystemColors.WindowTextBrush;

                if(targetObject is Window)
                {
                    this.window = targetObject as Window;

                    this.parseStatusBarItem.Content = "Press F7 to display Window";
                }
                else
                {
                    this.window = null;

                    this.frame.Content = targetObject;

                    this.parseStatusBarItem.Content = "OK";
                }
            }
            catch(Exception exception)
            {
                this.textBox.Foreground = Brushes.Red;

                this.parseStatusBarItem.Content = exception.Message;
            }
        }

        #endregion
    }
}