﻿using System;
using System.Globalization;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// XAML 방향 메뉴 항목
    /// </summary>
    public class XamlOrientationMenuItem : MenuItem
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 체크 하위 메뉴 항목
        /// </summary>
        private MenuItem checkedSubMenuItem;

        /// <summary>
        /// 그리드
        /// </summary>
        private Grid grid;

        /// <summary>
        /// 텍스트 박스
        /// </summary>
        private TextBox textBox;

        /// <summary>
        /// 프레임
        /// </summary>
        private Frame frame;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 방향 - Orientation

        /// <summary>
        /// 방향
        /// </summary>
        public Dock Orientation
        {
            set
            {
                foreach(MenuItem menuItem in Items)
                {
                    if(menuItem.IsChecked = (value == (Dock)menuItem.Tag))
                    {
                        this.checkedSubMenuItem = menuItem;
                    }
                }
            }
            get
            {
                return (Dock)this.checkedSubMenuItem.Tag;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - XamlOrientationMenuItem(grid, textBox, frame)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="grid">Grid 객체</param>
        /// <param name="textBox">TextBox 객체</param>
        /// <param name="frame">Frame 객체</param>
        public XamlOrientationMenuItem(Grid grid, TextBox textBox, Frame frame)
        {
            this.grid    = grid;
            this.textBox = textBox;
            this.frame   = frame;

            Header = "_Orientation";

            for(int i = 0; i < 4; i++)
            {
                Items.Add(CreateSubMenuItem((Dock)i));
            }

            (this.checkedSubMenuItem = (MenuItem)Items[0]).IsChecked = true;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 하위 메뉴 항목 클릭시 처리하기 - subMenuItem_Click(sender, e)

        /// <summary>
        /// 하위 메뉴 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void subMenuItem_Click(object sender, RoutedEventArgs e)
        {
            this.checkedSubMenuItem.IsChecked = false;

            this.checkedSubMenuItem = e.Source as MenuItem;

            this.checkedSubMenuItem.IsChecked = true;
        }

        #endregion
        #region 하위 메뉴 항목 체크시 처리하기 - pSubMenuItem_Checked(sender, e)

        /// <summary>
        /// 하위 메뉴 항목 체크시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void subMenuItem_Checked(object sender, RoutedEventArgs e)
        {
            MenuItem checkedSubMenuItem = e.Source as MenuItem;

            for(int i = 1; i < 3; i++)
            {
                this.grid.RowDefinitions[i].Height = new GridLength(0);

                this.grid.ColumnDefinitions[i].Width = new GridLength(0);
            }

            Grid.SetRow   (this.textBox, 0);
            Grid.SetColumn(this.textBox, 0);

            Grid.SetRow   (this.frame, 0);
            Grid.SetColumn(this.frame, 0);

            switch((Dock)checkedSubMenuItem.Tag)
            {
                case Dock.Left :

                    this.grid.ColumnDefinitions[1].Width = GridLength.Auto;
                    this.grid.ColumnDefinitions[2].Width = new GridLength(100, GridUnitType.Star);

                    Grid.SetColumn(this.frame, 2);

                    break;

                case Dock.Top :

                    this.grid.RowDefinitions[1].Height = GridLength.Auto;
                    this.grid.RowDefinitions[2].Height = new GridLength(100, GridUnitType.Star);

                    Grid.SetRow(this.frame, 2);

                    break;

                case Dock.Right :

                    this.grid.ColumnDefinitions[1].Width = GridLength.Auto;
                    this.grid.ColumnDefinitions[2].Width = new GridLength(100, GridUnitType.Star);

                    Grid.SetColumn(this.textBox, 2);

                    break;

                case Dock.Bottom :

                    this.grid.RowDefinitions[1].Height = GridLength.Auto;
                    this.grid.RowDefinitions[2].Height = new GridLength(100, GridUnitType.Star);

                    Grid.SetRow(this.textBox, 2);

                    break;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region FormattedText 객체 구하기 - GetFormattedText(source)

        /// <summary>
        /// FormattedText 객체 구하기
        /// </summary>
        /// <param name="source">소스 문자열</param>
        /// <returns>FormattedText 객체</returns>
        private FormattedText GetFormattedText(string source)
        {
            return new FormattedText
            (
                source,
                CultureInfo.CurrentCulture,
                FlowDirection.LeftToRight,
                new Typeface(SystemFonts.MenuFontFamily, SystemFonts.MenuFontStyle, SystemFonts.MenuFontWeight, FontStretches.Normal),
                SystemFonts.MenuFontSize,
                SystemColors.MenuTextBrush
            );
        }

        #endregion
        #region 박스 텍스트 그리기 - DrawBoxText(drawingContext, formattedText, width, location)

        /// <summary>
        /// 박스 텍스트 그리기
        /// </summary>
        /// <param name="drawingContext">DrawingContext 객체</param>
        /// <param name="formattedText">FormattedText 객체</param>
        /// <param name="width">너비</param>
        /// <param name="location">위치</param>
        private void DrawBoxText(DrawingContext drawingContext, FormattedText formattedText, double width, Point location)
        {
            Pen pen = new Pen(SystemColors.MenuTextBrush, 1);

            drawingContext.DrawRectangle(null, pen, new Rect(location.X, location.Y, width + 4, formattedText.Height + 4));

            double x = location.X + (width - formattedText.Width) / 2;

            drawingContext.DrawText(formattedText, new Point(x + 2, location.Y + 2));
        }

        #endregion
        #region 하위 메뉴 항목 생성하기 - CreateSubMenuItem(dock)

        /// <summary>
        /// 하위 메뉴 항목 생성하기
        /// </summary>
        /// <param name="dock">Dock 객체</param>
        /// <returns>하위 메뉴 항목</returns>
        private MenuItem CreateSubMenuItem(Dock dock)
        {
            MenuItem subMenuItem = new MenuItem();

            subMenuItem.Tag = dock;

            subMenuItem.Click   += subMenuItem_Click;
            subMenuItem.Checked += subMenuItem_Checked;

            FormattedText editFormattedText    = GetFormattedText("Edit"   );
            FormattedText displayFormattedText = GetFormattedText("Display");

            double maximumWidth = Math.Max(editFormattedText.Width, displayFormattedText.Width);

            DrawingVisual  drawingVisual  = new DrawingVisual();
            DrawingContext drawingContext = drawingVisual.RenderOpen();

            switch(dock)
            {
                case Dock.Left :

                    DrawBoxText(drawingContext, editFormattedText   , editFormattedText.Width   , new Point(0                          , 0));
                    DrawBoxText(drawingContext, displayFormattedText, displayFormattedText.Width, new Point(editFormattedText.Width + 4, 0));

                    break;

                case Dock.Top :

                    DrawBoxText(drawingContext, editFormattedText   , maximumWidth, new Point(0, 0                           ));
                    DrawBoxText(drawingContext, displayFormattedText, maximumWidth, new Point(0, editFormattedText.Height + 4));

                    break;

                case Dock.Right :

                    DrawBoxText(drawingContext, displayFormattedText, displayFormattedText.Width, new Point(0                             , 0));
                    DrawBoxText(drawingContext, editFormattedText   , editFormattedText.Width   , new Point(displayFormattedText.Width + 4, 0));

                    break;

                case Dock.Bottom :

                    DrawBoxText(drawingContext, displayFormattedText, maximumWidth, new Point(0, 0                              ));
                    DrawBoxText(drawingContext, editFormattedText   , maximumWidth, new Point(0, displayFormattedText.Height + 4));

                    break;
            }

            drawingContext.Close();

            DrawingImage drawingImage = new DrawingImage(drawingVisual.Drawing);

            Image image = new Image();

            image.Source = drawingImage;

            subMenuItem.Header = image;

            return subMenuItem;
        }

        #endregion
    }
}